#include "config.h"

#include <xgislib/osgutils.h>
#include <xgislib/xputils.h>
#include <xgislib/quad.h>

#include <string.h>

#include "layer-text.h"

#define FLOAT_VAL_UNDEF 1e30

struct LayerTextPluginContext : public XGFLoaderPluginContext {
	float x, y;
	std::string text;
	std::string tmp;

	osg::Node *createNamedLocGeode(const std::string &name, float x, float y, XGFLoaderContext *lb) {
		osg::Vec4 layoutColor(0.0f,0.0f,0.0f,1.0f);
		float layoutCharacterSize = 30.0f;		
		float z = lb->getWorld()->getTerrainHeight(x, y) + 100.0; //FIXME: make this height a parameter
		
		DeferredText *text = new DeferredText(name.c_str(),
				osgText::String::ENCODING_UTF8);
		text->setFont(lb->getResourceBag()->getFont(FONT_TIMES));
		text->setAxisAlignment(osgText::Text::SCREEN);
		text->setColor(layoutColor);
		text->setCharacterSize(layoutCharacterSize);
		text->setPosition(osg::Vec3(x, y, z));
		text->setLayout(osgText::Text::LEFT_TO_RIGHT);
		text->setDrawMode(osgText::Text::TEXT);
		text->setCharacterSizeMode(osgText::Text::
			OBJECT_COORDS_WITH_MAXIMUM_SCREEN_SIZE_CAPPED_BY_FONT_HEIGHT);
		text->setAlignment(osgText::Text::CENTER_CENTER);
		text->setMaximumHeight(30);

		osg::Geode *rv = new osg::Geode;
		rv->addDrawable(text);
		return rv;
	}	
};

LayerTextPlugin::LayerTextPlugin() : XGFLoaderPlugin() {
	sax.startElement = LayerTextPlugin::startElement;
	sax.endElement = LayerTextPlugin::endElement;
	sax.characters = LayerTextPlugin::characters;
}

void LayerTextPlugin::startElement(void *ctx, 
	const xmlChar *name, 
	const xmlChar **atts) {
	DECLARE_XGFLOADER_PLUGIN_CTX(tp, LayerTextPluginContext);

	if(0 == strcmp((const char *)name, "namedlocation")) {
		tp->x = tp->y = FLOAT_VAL_UNDEF;
		tp->text = "";
	}
	tp->tmp = "";
}

void LayerTextPlugin::endElement(void *ctx, 
	const xmlChar *name) {
	DECLARE_XGFLOADER_CTX(lb);
	DECLARE_XGFLOADER_PLUGIN_CTX(tp, LayerTextPluginContext);

	if(0 == strcmp((const char *)name, "namedlocation")) {
		XGIS_LOG_DEBUG5("Creating text at (%f,%f): %s",
			tp->x, tp->y, tp->text.c_str());
		if(tp->text.length() > 0 &&
			tp->x != FLOAT_VAL_UNDEF && tp->y != FLOAT_VAL_UNDEF) {
			osg::Node *txt = tp->createNamedLocGeode(tp->text, tp->x, tp->y, lb);
			if(NULL != txt) {
				lb->addResult(txt);
			}
		}
	}
	else if(0 == strcmp((const char *)name, "name")) {
		tp->text = tp->tmp;
	}
	else if(0 == strcmp((const char *)name, "location")) {
		sscanf(tp->tmp.c_str(), "%f,%f", &(tp->x), &(tp->y));
	}

}

void LayerTextPlugin::characters(void *ctx, 
	const xmlChar *ch, 
	int len) {
	DECLARE_XGFLOADER_PLUGIN_CTX(tp, LayerTextPluginContext);

	tp->tmp.insert(tp->tmp.length(), (const char *)ch, len);
}

const std::string &LayerTextPlugin::element() {
	static std::string e = "namedlocation";
	return e;
}

XGFLoaderPluginContext *LayerTextPlugin::createPluginContext() {
	return new LayerTextPluginContext(); 
}
