#ifndef __LOADERS_XGF_XGFLOADER_H__
#define __LOADERS_XGF_XGFLOADER_H__

#include <libxml/xmlreader.h>

#include <osg/Group>

#include <xgislib/coords.h>
#include <xgislib/resourcebag.h>
#include <xgislib/layerset.h>
#include <xgislib/layerbuilder.h>

class XGFLoaderPluginContext {
	// perhaps someting, someday
};

class XGFLoaderPlugin {
	friend class XGFLoader;

protected:	
	xmlSAXHandler sax;
	
public:
	XGFLoaderPlugin();
	virtual const std::string &element() = 0;
	virtual XGFLoaderPluginContext *createPluginContext() = 0;
};

class XGFLoaderContext {
	friend class XGFLoaderPlugin;
	friend class XGFLoader;

protected:
	LayerLoaderContext *ctx;

	XGFLoaderPlugin *currentPlugin;
	XGFLoaderPluginContext *pluginContext;
	
	int depth;
	std::vector<std::string> elementStack;
	
	osg::Group *layerNodes;
	std::vector<LayerEventHandlers *> layerEventHandlers;
	
	const std::vector<XGFLoaderPlugin *> &plugins;

public:
	XGFLoaderContext(const std::vector<XGFLoaderPlugin *> &_plugins,
		LayerLoaderContext *_ctx);

	XGFLoaderPlugin *getCurrentPlugin() {
		return currentPlugin;
	}

	XGFLoaderPluginContext *getPluginContext() {
		return pluginContext;
	}
	
	World *getWorld() {
		return ctx->getWorld();
	}
	
	ResourceBag *getResourceBag() {
		return ctx->getResourceBag();
	}
	
	WorldStorage *getStorage() {
		return ctx->getStorage();
	}
	
    void addResult(osg::Node *rnode) {
    	layerNodes->addChild(rnode);
    }
    
    void addResult(LayerEventHandlers *handlers) {
    	layerEventHandlers.push_back(handlers);
    }
};

class XGFLoader : public LayerLoader {
private:
	xmlSAXHandler sax;
	
	std::vector<XGFLoaderPlugin *> plugins;

protected:
    static void startDocument(void *ctx);
    static void endDocument(void *ctx);
    static void startElement(void *ctx, 
		const xmlChar *name, 
		const xmlChar **atts);
    static void endElement(void *ctx, 
    	const xmlChar *name);
    static void characters(void *ctx, 
    	const xmlChar *ch, 
    	int len);
    static void ignorableWhitespace(void * ctx, 
    	const xmlChar *ch, 
    	int len);
    static void warning(void *ctx, 
    	const char *msg, 
    	...);
    static void error(void *ctx, 
    	const char *msg, 
    	...);
    static void fatalError(void *ctx, 
        const char * msg, 
        ...);
    static void cdataBlock(void *ctx, 
    	const xmlChar *value, 
    	int len);
    static void startElementNs(void *ctx, 
		const xmlChar *localname, 
		const xmlChar *prefix, 
		const xmlChar *URI, 
		int nb_namespaces, 
		const xmlChar **namespaces, 
		int nb_attributes, 
		int nb_defaulted, 
		const xmlChar **attributes);
    static void endElementNs(void *ctx, 
		const xmlChar *localname, 
		const xmlChar *prefix, 
		const xmlChar *URI);
    static void serror(void *ctx, 
    	xmlErrorPtr error);    

public:
	XGFLoader();
	
	virtual Layer *loadLayer(const std::string &file, LayerLoaderContext *ctx);
	
	bool registerPlugin(XGFLoaderPlugin *plugin);
	XGFLoaderPlugin *unregisterPlugin(const std::string &element);	
};

// use this to declare a layer builder in SAX callbacks
#define DECLARE_XGFLOADER_CTX(lb) XGFLoaderContext *lb = (XGFLoaderContext *)ctx
#define DECLARE_XGFLOADER_PLUGIN_CTX(lb, typ) typ *lb = (typ *)((XGFLoaderContext *)ctx)->getPluginContext(); 

#endif /* __LOADERS_XGF_XGFLOADER_H__ */
