#include "config.h"

#include <stdlib.h>
#include <stdio.h>
#include <math.h>

const char *usage = "Usage: quadinfo <quad-file>\n";

int main(int argc, char **argv) {
	if(argc != 2) {
		fprintf(stderr, usage);
		return -1;
	}
	
	FILE *srcf = fopen(argv[1], "rb");
	if(NULL == srcf) {
		fprintf(stderr, "Failed to open quad file %s.\n", argv[1]);
		return -42;
	}

	int header[4];
	fread(header, sizeof(int), 4, srcf);
	
	printf("Source quad size: %d\n", header[0]);
	printf("Source quad resolution: %d\n", header[1]);
	printf("Source quad LL corner: (%d, %d)\n", header[2], header[3]);

	unsigned int pointsPerLine = header[0]/1000*header[1] + 1;
	printf("Points per source line: %d\n", pointsPerLine);

	int invalid = 0, total = 0;
	float *vline = new float[pointsPerLine];
	for(unsigned int i = 0; i < pointsPerLine; i++) {
		// DBG_PRINT("Reading vertical line %d...", i); fflush(stdout);
		if(pointsPerLine == fread(vline, sizeof(float), pointsPerLine, srcf)) {
			// DBG_PRINT(" ok!\n");
		}
		else {
			printf("Reading failed at line %d (of %d)! Quad file is corrupt!\n",
				i + 1, pointsPerLine);
			fclose(srcf);
			return -181;
		}
		
		for(unsigned int j = 0; j < pointsPerLine; j++) {
			if(isnan(vline[j]))
				invalid++;
			total++;
		}
	}
	delete vline;
	
	printf("Invalid heights (NaN) in quad: %d out of %d\n", invalid, total);
	
	int i = 0;
	while(-1 != fgetc(srcf)) i++;
	
	fclose(srcf);

	if(i > 0) {
		printf("Garbage (%d bytes) at quad file end. Quad file might be corrupt!\n", i);
		return -67;
	}
	
	return 0;
}
