#include <tiffio.h>
#include <stdio.h>

uint32 width, height;
uint32 tileW = 160;
uint32 tileH = 160;

void writeATiff() {
	TIFF *output;
	char *raster;
	
	// Open the output image
	if((output = TIFFOpen("output.tif", "w")) == NULL) {
		fprintf(stderr, "Could not open outgoing image\n");
		exit(42);
	}

	// We need to know the width and the height before we can malloc
	if ((raster = (char *) malloc(sizeof(char) * tileW * tileH)) == NULL) {
		fprintf(stderr, "Could not allocate enough memory\n");
		exit(42);
	}

	// Magical stuff for creating the image
	for (int i = 0; i < tileW*tileH; i++)
		raster[i] = 255;

	// Write the tiff tags to the file
	TIFFSetField(output, TIFFTAG_IMAGEWIDTH, width);
	TIFFSetField(output, TIFFTAG_IMAGELENGTH, height);
	TIFFSetField(output, TIFFTAG_TILEWIDTH, tileW);
	TIFFSetField(output, TIFFTAG_TILELENGTH, tileH);
	TIFFSetField(output, TIFFTAG_COMPRESSION, COMPRESSION_LZW);
	TIFFSetField(output, TIFFTAG_PLANARCONFIG, PLANARCONFIG_CONTIG);
	TIFFSetField(output, TIFFTAG_PHOTOMETRIC, PHOTOMETRIC_MINISBLACK);
	TIFFSetField(output, TIFFTAG_BITSPERSAMPLE, 8);
	TIFFSetField(output, TIFFTAG_SAMPLESPERPIXEL, 1);
	
	TIFFWriteTile(output, raster, 4*tileW, 5*tileH, 0, 0);

	// Actually write the image
	for (int x = 0; x < width; x += tileW)
		for (int y = 0; y < height; y += tileH)
			TIFFWriteTile(output, raster, x, y, 0, 0);

	for (int i = 0; i < tileW*tileH; i++)
		raster[i] = i % 255;
	TIFFWriteTile(output, raster, 4*tileW, 5*tileH, 0, 0);

	TIFFClose(output);
}

void changeATiff() {
	TIFF *tif;
	char *raster;
	
	// Open the output image
	if((tif = TIFFOpen("output.tif", "a")) == NULL) {
		fprintf(stderr, "Could not open outgoing image\n");
		exit(42);
	}

	// We need to know the width and the height before we can malloc
	if ((raster = (char *) malloc(sizeof(char) * tileW * tileH)) == NULL) {
		fprintf(stderr, "Could not allocate enough memory\n");
		exit(42);
	}

	// Magical stuff for creating the image
	for (int i = 0; i < tileW*tileH; i++)
		raster[i] = 200 - (i % 100);

	for (int x = 0; x < width; x += tileW)
		for (int y = 0; y < height; y += tileH)
			TIFFWriteTile(tif, raster, x, y, 0, 0);

	//TIFFWriteTile(tif, raster, 2*tileW, 5*tileH, 0, 0);

	TIFFClose(tif);

}

int main(int argc, char *argv[]) {
	width = tileW*10;
	height = tileH*10;

	//writeATiff();
	changeATiff();
}