#include <config.h>
#include "stdlib.h"
#include <xgislib/xgislog.h>

#include <xgislib/cirrus.h>
#include <xgislib/pointdata.h>
#include <xgislib/xgisimgutils.h>

#include <osg/BlendFunc>
#include <osg/TexGen>
#include <osg/Material>
#include <osg/PointSprite>
#include <osg/Point>
#include <osg/Image>
#include <osg/StateSet>
#include <osg/Texture2D>

Cirrus::Cirrus(WorldStorage *_storage, int _numParticles, int _partSize, 
			   float _sizeX, float _sizeY, float _thick, const osg::Vec3f &_sunPos, 
			   time_t _startTime) 
: Cloud(_storage, _numParticles, _partSize, _sizeX, _sizeY, _thick,_sunPos, 
		_startTime) {

//	osg::StateSet *cloudState = this->getOrCreateStateSet();
	//cloudState->setRenderingHint(osg::StateSet::RenderingHint::TRANSPARENT_BIN);
	placeParticles();
}

osg::Vec4f Cirrus::getCirrusColour(float altitude) {
	float c1 = 8.0;
	float c2 = 0.0;
	float goal = 0.3;

	osg::Vec4f ret;
	float val = 1.0;

	if (altitude < c1)  {
		val -= (c1 - altitude)/(c1 - c2) * (val - goal);
	}

	val = MAX(val, goal);

	ret = osg::Vec4f(val, val, val, 1.0);

	return ret;
}

void Cirrus::placeParticles() {

	osg::Geometry *cloud = new osg::Geometry();
	osg::Vec3Array *vertices = new osg::Vec3Array();
	osg::Vec4Array *colours = new osg::Vec4Array();

	double azimuth, altitude;
	Coords::GaussKr pos(sunPosition.x(), sunPosition.y());

	Pointdata::FeatureInfo::getSunPosition(pos, startTime, azimuth, altitude);

	osg::Vec4f colour = getCirrusColour(altitude);

	colours->push_back(colour);

	osg::ref_ptr<osg::Geode> cloudGeode = new osg::Geode();

	numParticles = 0;


	osg::ref_ptr<osg::Image> image = new osg::Image;
	
	int numPixels = 257;
	image->allocateImage(numPixels,numPixels,1,GL_RGBA,GL_FLOAT);
    image->setInternalTextureFormat(GL_RGBA);

	XGisImage::initImage(image.get());
	XGisImage::generateFractal(image.get(), 1.0, true, false, false);
	XGisImage::scaleImage(image.get(), numPixels-1, numPixels-1);
	

#if 0
	for (int i=0; i<image.get()->s(); i++) {
		for (int j=0; j<image.get()->t(); j++) {
			osg::Vec4f *data = (osg::Vec4f*)image->data(i,j);
			XGIS_LOG_DEBUG2("%f %f",data->r(), data->a());
		}
	
	}
#endif

	XGisImage::squareRootImage(image.get());
	XGisImage::clampImage(image.get(), 0.6);
	XGisImage::smoothBorder(image.get(), 80, true, 0);
	//XGisImage::makeCircle(image.get());
	
	

	//convolve(image, 0);
	

	osg::Texture2D *cirrusTexture = new osg::Texture2D();
	cirrusTexture->setWrap(osg::Texture2D::WRAP_S, osg::Texture2D::CLAMP_TO_EDGE);
	cirrusTexture->setWrap(osg::Texture2D::WRAP_T, osg::Texture2D::CLAMP_TO_EDGE);

	cirrusTexture->setUnRefImageDataAfterApply(true);
	cirrusTexture->setImage(image.get());
	

	vertices->push_back(osg::Vec3f(0,0,0));
	vertices->push_back(osg::Vec3f(sizeX,0,0));
	vertices->push_back(osg::Vec3f(sizeX,sizeY,0));
	vertices->push_back(osg::Vec3f(0,sizeY,0));

	osg::Vec2Array* texCoords = new osg::Vec2Array(4);
   (*texCoords)[0].set(0.0f,0.0f);
   (*texCoords)[1].set(1.0f,0.0f);
   (*texCoords)[2].set(1.0f,1.0f);
   (*texCoords)[3].set(0.0f,1.0f);
   cloud->setTexCoordArray(0,texCoords);

 /*     osg::DrawElementsUInt* q = 
      new osg::DrawElementsUInt(osg::PrimitiveSet::QUADS, 0);
   q->push_back(3);
   q->push_back(2);
   q->push_back(1);
   q->push_back(0);
   cloud->addPrimitiveSet(q); */

   osg::StateSet *cloudState = cloud->getOrCreateStateSet();

   cirrusTexture->setFilter(osg::Texture2D::MIN_FILTER,
	   osg::Texture2D::LINEAR_MIPMAP_LINEAR);
   cirrusTexture->setFilter(osg::Texture2D::MAG_FILTER,
	   osg::Texture2D::LINEAR_MIPMAP_LINEAR);
   
   cloudState->setTextureAttribute(0,cirrusTexture,osg::StateAttribute::ON);
	cloudState->setMode(GL_LIGHTING, osg::StateAttribute::OFF);
	cloudState->setMode(GL_DEPTH_TEST, osg::StateAttribute::ON);   

   cloudState->setTextureMode(0, GL_TEXTURE_2D, osg::StateAttribute::ON );



	/*this makes... not much
	I need to try with textures*/
#if 0
	float unit = (float)particleSize;//((float)particleSize/4.0);
	float unitX = (float)particleSize/5.0;//((float)particleSize/4.0);

	float i = 0;
	float j = 0;
	float c = 0;
	float c1 = 0;
	while (i < sizeX) {
		unit = (float)particleSize/5.0;
		j = 0;
		c = 0;
		while (j < sizeY) {
			XGIS_LOG_DEBUG3("unit %f",unit);
			float p = ((rand() % 100)/100.0) - 0.5;
			float q = ((rand() % 100)/100.0) - 0.5;

			float offX = i;
			float offY = j+p*unitX*20.0;
			float offZ = 0;

			numParticles++;

			float opacity = (cos(osg::PI * (i/sizeX - 0.5))*cos(osg::PI 
				* (j/sizeY - 0.5)) - 0.3)/5.0;

			XGIS_LOG_DEBUG3("%f %f %f %f",offX, offY,offZ, opacity);
			
			vertices->push_back(osg::Vec3f(offX,offY,offZ));
			colors->push_back(osg::Vec4f(1.0,1.0,1.0,opacity));
			j += unit;
			c++;
			unit+=10;

		}
		c1++;
		i += unitX;
		//unitX += exp(c/1000.0);
	}
#endif
	/*this makes a spritey blob across the sky*/
#if 0
	float unit = 4.0*(float)particleSize;//((float)particleSize/4.0);

	for(float i=0; i < sizeX; i+= unit) {
		for (float j=0; j < sizeY; j+= unit) {

			float p = ((rand() % 100)/100.0) - 0.5;
			float q = ((rand() % 100)/100.0) - 0.5;

			float offX = i + p*unit;
			float offY = j + q*unit;
			float offZ = 0;

			numParticles++;

			float opacity = cos(fabs(q)*osg::PI)*cos(fabs(p)*osg::PI) - 0.2;
			XGIS_LOG_DEBUG3("%f %f %f %f",offX, offY,offZ, opacity);
			
			vertices->push_back(osg::Vec3f(offX,offY,offZ));
			colors->push_back(osg::Vec4f(1.0,1.0,1.0,opacity));
		}
	}
#endif
	/*this makes a fluffy blob with a big particleSize*/
#if 0
	for(int i=0; i < sizeX; i+= unit) {
		for (int j=0; j < sizeY; j+= unit) {

			float p = ((rand() % 100)/100.0) - 0.5;
			float q = ((rand() % 100)/100.0) - 0.5;

			float offX = i;
			float offY = j;
			float offZ = 0;

			numParticles++;

			float opacity = cos(fabs(q)*osg::PI)*cos(fabs(p)*osg::PI) - 0.2;
			XGIS_LOG_DEBUG2("%f %f %f %f",offX, offY,offZ, opacity);
			
			vertices->push_back(osg::Vec3f(offX,offY,offZ));
			colors->push_back(osg::Vec4f(1.0,1.0,1.0,opacity));
		}
	}
#endif	

	cloud->setVertexArray(vertices);
	cloud->setColorArray(colours);
	cloud->setColorBinding(osg::Geometry::BIND_OVERALL);
	cloud->addPrimitiveSet(new osg::DrawArrays(osg::PrimitiveSet::QUADS, 0,4));

	cloudGeode->addDrawable(cloud);
	this->addChild(cloudGeode.get());
}
