#include <config.h>
#include "stdlib.h"
#include <xgislib/xgislog.h>

#include <xgislib/cloud.h>

#include <osg/Fog>
#include <osg/BlendFunc>
#include <osg/Depth>
#include <osg/PointSprite>
#include <osg/Point>
#include <osgDB/ReadFile>
#include <osg/Texture2D>

osg::Vec3f Cloud::getLightColour(float altitude){
	osg::Vec3f ret;
	int c1 = 5; int c3 = -7;
	//return osg::Vec3f(1.0, 1.0, 1.0);

	if (altitude >= c1) {
/*day. white*/
		ret = osg::Vec3f(1.0, 1.0, 1.0);

	} else if ((altitude < c1) && (altitude >= c3)) {
/*sunrise / sunset. yellow*/
		ret = osg::Vec3f(1.0, 0.95, 0.80);
	} else {
/*night.gray.*/	
		ret = osg::Vec3f(0.8, 0.8, 0.8);
	}

	return ret;
}

void Cloud::placeParticles() {
	
	osg::Geometry *cloud = new osg::Geometry();
	osg::Vec3Array *vertices = new osg::Vec3Array();
	osg::Vec4Array *colors = new osg::Vec4Array();

	for (int j=0;j<numParticles;j++) {

		float offX = ((rand() % 100)/100.0 * sizeX) - sizeX/2.0;
		if (offX < sizeX/3.0) {offX = offX/3.0; }
		
		float offY =((rand() % 100)/100.0 * sizeY) - sizeY/2.0;
		if (offY < sizeY/3.0) {offY = offY/3.0; }

		float offZ =((rand() % 100)/100.0 * thickness) - thickness/2.0;
		if (offZ < thickness/3.0) {offZ = offZ/3.0; }


		vertices->push_back(osg::Vec3f(offX,offY,offZ));
		colors->push_back(osg::Vec4f(1,1,1,0.5));
	}

	cloud->setVertexArray(vertices);
	cloud->setColorArray(colors);
	cloud->setColorBinding(osg::Geometry::BIND_PER_VERTEX);
	cloud->addPrimitiveSet(new osg::DrawArrays(osg::PrimitiveSet::POINTS, 0, 
		numParticles));
	osg::ref_ptr<osg::Geode> cloudGeode = new osg::Geode();

	cloudGeode->addDrawable(cloud);
	this->addChild(cloudGeode.get());

}

Cloud::Cloud(WorldStorage *_storage, int _numParticles, int _partSize,
	float _sizeX, float _sizeY, float _thick, const osg::Vec3f &_sunPos,
	time_t _startTime) {
	numParticles = _numParticles;
	particleSize = _partSize;
	sizeX = _sizeX;
	sizeY = _sizeY;
	thickness = _thick;
	sunPosition = _sunPos;
	startTime = _startTime;
	storage = _storage;
	
	osg::StateSet *cloudState = new osg::StateSet();

	cloudState->setMode(GL_BLEND, osg::StateAttribute::ON);
	osg::BlendFunc *fn = new osg::BlendFunc();
	fn->setFunction(osg::BlendFunc::SRC_ALPHA, osg::BlendFunc::ONE_MINUS_SRC_ALPHA);
	cloudState->setAttributeAndModes(fn, osg::StateAttribute::ON);
	
	cloudState->setMode(GL_LIGHTING, osg::StateAttribute::OFF);
	cloudState->setMode(GL_DEPTH_TEST, osg::StateAttribute::ON);


	osg::Depth* depth = new osg::Depth();
	depth->setWriteMask(false);
	cloudState->setAttributeAndModes(depth, osg::StateAttribute::ON );
	
	//cloudState->setRenderingHint(osg::StateSet::RenderingHint::TRANSPARENT_BIN);

	this->setStateSet(cloudState);
}

Cloud::~Cloud() {	
}

#if 0
osg::BoundingBox Cloud::computeBound() const
{
	osg::BoundingBox bBox;
	std::vector<CloudParticle*>::iterator it;
	float minx = 99999, miny = 99999, minz = 99999, maxx = 0, maxy = 0, maxz = 0;
	float maxRad = 0;

	for (it = particles->begin(); it != particles->end(); it++) {
		CloudParticle *cp = (*it);
		if (cp->center.x() < minx) {minx = cp->center.x();}
		if (cp->center.x() > maxx) {maxx = cp->center.x();}
		if (cp->center.y() < miny) {miny = cp->center.y();}
		if (cp->center.y() > maxy) {maxy = cp->center.y();}
		if (cp->center.z() < minz) {minz = cp->center.z();}
		if (cp->center.z() > maxz) {maxz = cp->center.z();}

		if (cp->radius > maxRad) {maxRad = cp->radius;}
	}

	printf("%f %f %f %f %f %f\n",minx, miny, minz, maxx, maxy, maxz);
	printf("%f\n",maxRad);

	bBox.set(minx - maxRad, miny - maxRad, minz - 1, maxx + maxRad, maxy + maxRad, maxz + 1);

	return bBox;
}


void Cloud::addParticle(const osg::Vec3f &center, const osg::Vec4f &colour,
	float density, float radius) {
	CloudParticle *c = new CloudParticle(center, colour, density, radius);
	particles->push_back(c);
}


/*here we draw. sort of.*/
void Cloud::drawImplementation(osg::RenderInfo &renderInfo) const {
	/*get texture!*/
	glEnable(GL_TEXTURE_2D);
	glEnable(GL_BLEND);
	glDisable(GL_LIGHTING);	

	osg::Vec4f color = osg::Vec4f(1,1,1,1.0);

	GLfloat arr0[4] = {1.0, 1.0, 1.0, 1.0};
	
	GLuint texture[1];
	AUX_RGBImageRec *TextureImage[1];
	memset(TextureImage,0,sizeof(void *)*1);
	TextureImage[0] = auxDIBImageLoad(TEXT("../xgisviewer-wx/cloud1.bmp"));
	if (TextureImage[0]) {
		glGenTextures(1, &texture[0]);
		glBindTexture(GL_TEXTURE_2D, texture[0]);

		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);

		glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP );
		glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP );
		int sizeX = TextureImage[0]->sizeX;
		int sizeY = TextureImage[0]->sizeY;
		
		float *newData = (float*)malloc(sizeof(float)*sizeX*sizeY*4);
		for (int i=0;i<sizeX;i++) {
			for (int j=0;j<sizeY;j++) {
				//XGIS_LOG_DEBUG2("%d %d %f",i*sizeY*3 + j*3,i*sizeY*4 + j*4,TextureImage[0]->data[i*sizeY + j]/255.0);

				newData[i*sizeY*4 + j*4] = TextureImage[0]->data[i*sizeY*3 + j*3]/255.0;
				newData[i*sizeY*4 + j*4 + 1] = TextureImage[0]->data[i*sizeY*3 + j*3 + 1]/255.0;
				newData[i*sizeY*4 + j*4 + 2] = TextureImage[0]->data[i*sizeY*3 + j*3 + 2]/255.0;
				newData[i*sizeY*4 + j*4 + 3] = TextureImage[0]->data[i*sizeY*3 + j*3]/255.0;
			}
		}

		//glTexImage2D(GL_TEXTURE_2D, 0, 3, TextureImage[0]->sizeX, TextureImage[0]->sizeY, 0, GL_RGB, GL_UNSIGNED_BYTE, TextureImage[0]->data);
		glTexImage2D(GL_TEXTURE_2D, 0, 4, TextureImage[0]->sizeX, TextureImage[0]->sizeY, 0, GL_RGBA, GL_FLOAT, newData);

		if (TextureImage[0])							
		{
			if (TextureImage[0]->data)					
			{
				free(TextureImage[0]->data);
				//free(newData);
			}

			free(TextureImage[0]);						
		}
	} else {
		XGIS_LOG_DEBUG2("Texture not loaded!");
	}

	std::vector<CloudParticle*>::iterator it;

	
	//glBlendFunc(GL_SRC_COLOR, GL_ONE_MINUS_SRC_COLOR);	
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);	

	for (it = particles->begin(); it != particles->end(); it++) {
		CloudParticle *cp = (*it);

		osg::Vec3f pos = cp->center;
		osg::Vec3f x = osg::Vec3f(1,0,0)*cp->radius;
		osg::Vec3f y = osg::Vec3f(0,1,0)*cp->radius;

		glBegin(GL_QUADS);

		glColor4fv(&(color.x()));

		osg::Vec3f left  = pos;  left   -= y; 
		osg::Vec3f right = left; right  += x; 
		left  -= x;
		glTexCoord2f(0, 0); glVertex3fv(&(left.x()));
		glTexCoord2f(1, 0); glVertex3fv(&(right.x()));
		left  += y;  left  += y;
		right += y;  right += y;
		glTexCoord2f(1, 1); glVertex3fv(&(right.x()));
		glTexCoord2f(0, 1); glVertex3fv(&(left.x()));

		glEnd();
	
	}
	glDisable(GL_BLEND);
	glEnable(GL_LIGHTING);	

}
#endif
