#include "config.h"

#include "coords.h"
#include "xputils.h"

#include <assert.h>
#include <math.h>
#include <string.h>

#include <proj4/cs2cs.h>

namespace Coords {

	const GaussKr GaussKr::reference = GaussKr(365000, 25000);

	const WGS84 WGS84::reference = WGS84(13.27471, 45.35843);

	GaussKr WGS84::toGaussKr() {
		GaussKr rv(0, 0);
		Cs2cs::wgs84ToGaussKrSlo(longit, latit, rv.x, rv.y);
		return rv;

		// old code, not accurate
		#ifdef XGIS_USE_THE_OLD_INACCURATE_COORDINATE_CONVERSION_FOR_WHATEVER_REASON_AND_NO_THIS_SYMBOL_IS_NOT_ACTUALLY_MEANT_TO_BE_DEFINED
			/* assuming coords in degrees*/
			
			double a = 6378137;       /*equatorial radius*/
			double b = 6356752.3142f; /*polar radius*/
			double n = (a-b)/(a+b);
			double n2 = n*n;
			double n3 = n2*n;
			double n4 = n3*n;

			double k0 = 0.9996f;
			double e = 0.08f;

			double latitRadians = latit*3.14;
			latitRadians = latitRadians/180.0;

			double p = (longit - 15.0f)*3600/10000.0f;
			double nu = a/sqrt(1-e*e*sin(latitRadians)*sin(latitRadians));
			double sin1s = M_PI/(180.0f*60.0f*60.0f);
			double eap = e*e/(1-e*e);
			
			double A = a*(1-n+(5*n*n/4.0f)*(1-n)+(81*pow(n,4)/64.0f)*(1-n));
			double B = (3*a*n/2.0f)*(1 - n + (7.0f*n2/8.0f)*(1 - n) +
				(55.0f/64.0f)*n4);
			double C = (15*a*n2/16.0f)*(1 - n + (3.0f*n2/4.0f)*(1 - n));
			double D = (35*a*n3/48.0f)*(1 - n + (11.0f/16.0f)*n2);
			double E = (315*a*n4/51.0f)*(1 - n);
			
			double S = A*latitRadians - B*sin(2*latitRadians) +
				C*sin(4*latitRadians) - D*sin(6*latitRadians) +
				E*sin(8*latitRadians);
			
			double K1 = S*k0;
			double K2 = nu*sin(latitRadians)*cos(latitRadians)*sin1s*sin1s*k0/
				2.0f*100000000;
			double K3 = ((pow(sin1s, 4)*nu*sin(latitRadians)*
				pow(cos(latitRadians), 3))/24.0f)*(5-pow(tan(latitRadians), 2) +
				9*eap*pow(cos(latitRadians), 2)+4*eap*eap*
				pow(cos(latitRadians), 4))*k0*(1e+016);
			  
			double northing = K1 + K2*pow(p, 2) + K3*pow(p, 4);
			
			double K4 = k0*sin1s*nu*cos(latitRadians)*10000;
			double K5 = k0*pow(sin1s, 3)*nu*pow(cos(latitRadians), 3)/
				6.0f*(1 - pow(tan(latitRadians),2) + eap*pow(cos(latitRadians),2));
			
			double easting = K4*p + K5*pow(p, 3);
			
			return GaussKr(floor(easting+500000), floor(northing-5000000));
		#endif /* XGIS_USE_THE_OLD_INACCURATE_COORDINATE_CONVERSION_FOR_WHATEVER_REASON_AND_NO_THIS_SYMBOL_IS_NOT_ACTUALLY_MEANT_TO_BE_DEFINED */
  }


	WGS84 GaussKr::toWGS84() {
		WGS84 rv(0, 0);
		Cs2cs::gaussKrSloTowgs84(x, y, rv.longit, rv.latit);
		return rv;

		// old code, not accurate
		#ifdef XGIS_USE_THE_OLD_INACCURATE_COORDINATE_CONVERSION_FOR_WHATEVER_REASON_AND_NO_THIS_SYMBOL_IS_NOT_ACTUALLY_MEANT_TO_BE_DEFINED
			/* constants */
			double k0 = 0.9996f;
			double e = 0.08f;
			double long0 = 0.261799f;
			
			double xnew = 500000.0f - x; /* easting from 15th meridian for
											Slovenia */
			double ynew = y+5000000;     /* northing from the equator*/
			
			double a = 6378137;       /* equatorial radius */
			#if 0
				/* currently unused var */
				double b = 6356752.3142f; /* polar radius */
			#endif /* 0/1 */
			
			double M = ynew/k0;
			double mu = M/(a*(1 - pow(e,2)/4.0f - 3*pow(e,4)/64.0f -
				5*pow(e,6)/256.0f));
			double e1 = (1 - sqrt(1 - e*e)) / (1 + sqrt(1 - e*e));
			
			double J1 = (3*e1/2.0f - 27*pow(e1,3)/32.0f);
			double J2 = (21*pow(e1,2)/16 - 55*pow(e1,4)/32.0f);
			double J3 = (151*pow(e1,3)/96.0f);
			double J4 = (1097*pow(e1,4)/512.0f);
			
			double fp = mu + J1*sin(2*mu) + J2*sin(4*mu) + J3*sin(6*mu) +
				J4*sin(8*mu);
			double eap = e*e/(1-e*e);
			double C1 = eap*pow(cos(fp),2);
			double T1 = pow(tan(fp),2);
			double R1 = a*(1-e*e)/pow((1-e*e*pow(sin(fp),2)), (double)1.5);
			double N1 = a/sqrt(1 - e*e*pow(sin(fp),2));
			double D = xnew/(N1*k0);
			
			double Q1 = N1*tan(fp)/R1;
			double Q2 = (D*D/2);
			double Q3 = (5 + 3*T1 + 10*C1 - 4*C1*C1 -9*eap)*pow(D,4)/24.0f;
			double Q4 = (61 + 90*T1 + 298*C1 +45*T1*T1  - 3*C1*C1 -252*eap)*
				pow(D,6)/720.0f;
			
			double Q5 = D;
			double Q6 = (1 + 2*T1 + C1)*pow(D,3)/6.0f;
			double Q7 = (5 - 2*C1 + 28*T1 - 3*C1*C1 + 8*eap + 24*T1*T1)*
				pow(D,5)/120.0f;
			
			double lat = fp - Q1*(Q2 - Q3 + Q4);
			//double longit = long0 + (Q5 - Q6 + Q7)/cos(fp);
			double longit = long0 - (Q5 - Q6 + Q7)/cos(fp);
			
			return WGS84(longit*180.0f/M_PI,lat*180.0f/M_PI);
		#endif /* XGIS_USE_THE_OLD_INACCURATE_COORDINATE_CONVERSION_FOR_WHATEVER_REASON_AND_NO_THIS_SYMBOL_IS_NOT_ACTUALLY_MEANT_TO_BE_DEFINED */
	}

	const double OrtoPhoto5::pixelSize = 0.5; //1 pixel is pixelSize meters
	const OrtoPhoto5 OrtoPhoto5::reference = OrtoPhoto5((const char *)"A1941", 0, 5999);

	bool OrtoPhoto5::isSheetValid(const char sheet[]) {
		//valid names are XYYss; X \in [A..L], YY \in [19..30], ss \in [1..50]
		if (strlen(sheet) != sheetNameLen)
			return false;
		if (sheet[0] < 'A' || sheet[0] > 'L')
			return false;
		
		char buf[] = "YY ss";
		buf[0] = sheet[1];
		buf[1] = sheet[2];
		buf[3] = sheet[3];
		buf[4] = sheet[4];
		
		int zoneY, sheetNum;
		if (sscanf(buf, "%d %d", &zoneY, &sheetNum) != 2)
			return false;
		if (zoneY < 19 || zoneY > 30)
			return false;
		if (sheetNum < 1 || sheetNum > 50)
			return false;

		return true;
	}

	OrtoPhoto5::OrtoPhoto5(const char *_sheet, int _u, int _v) {
		assert(isSheetValid(_sheet));
		strncpy(sheet, _sheet, sheetNameLen);
		u = _u;
		v = _v;
	}

	OrtoPhoto5::OrtoPhoto5(GaussKr gk) {
		static const int zoneW = 10, zoneH = 5;

		int x = int((gk.x - GaussKr::reference.x) / pixelSize);
		int y = int((gk.y - GaussKr::reference.y) / pixelSize);

		u = x % sheetW;
		v = (sheetH - 1) - (y % sheetH);

		int sheetX = x / sheetW, sheetY = y / sheetH;
		char zoneX = 'A' + (sheetX / zoneW);
		int zoneY = 19 + (sheetY / zoneH);
		int sheetNum = (1 + sheetX % zoneW) + 10*(4 - sheetY % zoneH);

		snprintf(sheet, sheetNameLen+1, "%c%02d%02d", zoneX, zoneY, sheetNum);

		assert(isSheetValid(sheet));
	}


	WGS84::WGS84(Coords::GaussKr gk) {
		longit = gk.toWGS84().longit;
		latit = gk.toWGS84().latit;
	}

}

