#ifndef __SOCALIB_COORDS_H__
#  define __SOCALIB_COORDS_H__

namespace Coords {
	
	class GaussKr;
	class WGS84;
	class OrtoPhoto5;

	// Austro-Hungarian Gauss-Krueger coordinates (as in relief model)
	class GaussKr {  
		friend class OrtoPhoto5;

	private:
		static const GaussKr reference; // reference point (LL-corner of
		                                // non-existing trig zone A19, which
		                                // would be Savudrija)
    
	public:
		double x, y; // offset in meters from a point somewhere in Italy
		             // x-axis points to east, y-axis to north
    
    	WGS84 toWGS84();

		GaussKr(double _x, double _y) {
			x = _x;
			y = _y;
		}
	};

	// longitude and latitue in decimal degrees
	class WGS84 {

	private:
		static const WGS84 reference; // reference point (LL-corner of
		                              // non-existing trig zone A19, which
		                              // would be Savudrija)

	public:
		double longit, latit; // positive means east longitude/north latitude

		WGS84(double _longit, double _latit) {
			longit = _longit;
			latit = _latit;
		}

		WGS84(GaussKr gk);

		GaussKr toGaussKr();
	};

	// coordinates of orto-photo 5 m (1 pixel = 0.5 m)
	class OrtoPhoto5 {
		friend class GaussKr;

	private:
		static const unsigned int sheetNameLen = 5;
		static const int sheetW = 4500, sheetH = 6000;
		static const double pixelSize;     // 1 pixel is pixelSize meters
		static const OrtoPhoto5 reference; // reference point (LL-corner of
		                                   // non-existing trig zone A19, which
		                                   // would be Savudrija)

		static bool isSheetValid(const char sheet[]);

	public:
		char sheet[sheetNameLen + 1];	// i.e. E2445
		int u, v;                       // pixel inside the sheet; u points to
		                                // east, v to south

		OrtoPhoto5(const char *_sheet, int _u, int _v);
		OrtoPhoto5(GaussKr gk);
	};
} 
#endif /* __SOCALIB_COORDS_H__ */
