#ifndef __XGISLIB_HTTPCLIENT_H__
#define __XGISLIB_HTTPCLIENT_H__

#include <string>

#include <OpenThreads/Mutex>

#ifndef XGIS_HTTP_STREAMING
#	include <libxml/nanohttp.h>
#else
#	ifdef WIN32
#		include <winsock2.h>
#	else
#   	include <sys/time.h>
#   	include <time.h>
#	endif /* WIN32 */
#endif /* !XGIS_HTTP_STREAMING */

#ifdef XGIS_HTTP_STREAMING
class HttpGetCallback {
public:
	virtual void handleStatus(int returnCode) = 0;
	virtual void handleData(const char *data, int dataSize) = 0;
};
#endif /* XGIS_HTTP_STREAMING */

typedef enum {
	SELECT_READ = 0,
	SELECT_WRITE = 1,
} SelectionType;

class HttpClient {
protected:	
	std::string server;
	unsigned short port;

#ifdef XGIS_HTTP_STREAMING
	OpenThreads::Mutex mutex;

#ifdef WIN32
	SOCKET sSock;
#else
	int sSock;
#endif /* WIN32 */
	char *buffer;

	int connect();
	int disconnect();
	bool isConnected();

	int select(SelectionType t);
	int read(char *buffer, int maxlen);

	int get(const std::string &path, HttpGetCallback *cb);

#ifdef XGIS_HTTP_CLIENT_THROTTLE		
	struct timeval tvs;
	unsigned long bytesInLastInterval;
#endif /* XGIS_HTTP_CLIENT_THROTTLE */
	
#else
	int fetch(const std::string &path, const std::string &fname);
#endif /* XGIS_HTTP_STREAMING */

public:
	HttpClient(const std::string &_server, unsigned short _port);
	~HttpClient();

	/* get returns:
	 *	-1 if opening connection fails
	 *  -2 if writing to destination file fails
	 *  HTTP return code otherwise (200 = OK, 404 = not found etc) */
	int get(const std::string &path, const std::string &fname);
};

#endif /* __XGISLIB_HTTPCLIENT_H__ */
