#include "config.h"

#include <stdlib.h>

#include <osg/Geode>
#include <osg/Geometry>
#include <osg/Group>
#include <osg/Node>
#include <osg/PositionAttitudeTransform>

#include <osgText/Text>
#include <osgText/Font>

#include <xgislib/layerbuilder.h>
#include <xgislib/xputils.h>
#include <xgislib/world.h>
#include <xgislib/quad.h>

LayerLoaderContext::LayerLoaderContext(World *_world,
	ResourceBag *_resourceBag, WorldStorage *_storage) :
		world(_world), resourceBag(_resourceBag), storage(_storage) {
}

LayerBuilder::LayerBuilder(World *_world, ResourceBag *_resourceBag,
	WorldStorage *_storage) :
		world(_world), resourceBag(_resourceBag), storage(_storage) {
}

void LayerBuilder::registerLoader(LayerLoader *loader) {
	loaders.push_back(loader);
}

Layer *LayerBuilder::buildLayer(const std::string &file, const std::string &mimeType) {

	// find a suitable loader
	std::vector<LayerLoader *>::const_iterator i = loaders.begin();
	LayerLoader *loader = NULL;
	
	while(i != loaders.end()) {
		if((*i)->canHandleMimeType(mimeType)) {
			loader = *i;
			break;
		}
	}
	
	if(NULL == loader) {
		// no loader can handle this mime type
		return NULL;
	}
	
	// if file is empty or does not exist, return an empty Group
	int fileSize = xpGetFileSize(file);
	if(fileSize <= 0) {
		return new Layer();
	}
	
	LayerLoaderContext *ctx = new LayerLoaderContext(world, resourceBag, storage);
	Layer *rv = loader->loadLayer(file, ctx);
	delete ctx;

	return rv;

}
