#ifndef __XGISLIB_LAYERBUILDER_H__
#define __XGISLIB_LAYERBUILDER_H__

#include <libxml/xmlreader.h>

#include <osg/Group>

#include <xgislib/coords.h>
#include <xgislib/resourcebag.h>
#include <xgislib/layerset.h>
#include <xgislib/quad.h>
#include <xgislib/resourcebag.h>
#include <xgislib/world.h>

class LayerLoaderContext {
protected:
	World *world;
	ResourceBag *resourceBag;	
	WorldStorage *storage;

public:
	LayerLoaderContext(World *_world, ResourceBag *_resourceBag, WorldStorage *_storage);

	World *getWorld() {
		return world;
	}
	
	ResourceBag *getResourceBag() {
		return resourceBag;
	}
	
	WorldStorage *getStorage() {
		return storage;
	}
	
};

class LayerLoader {
protected:
	std::string mimeType;    // mime type that is handled by this plugin

public:
	virtual bool canHandleMimeType(const std::string &typeToHandle) const {
		return (mimeType == typeToHandle);
	}
	
	virtual Layer *loadLayer(const std::string &file,
		LayerLoaderContext *ctx) = 0;
};

class LayerBuilder {
private:
	std::vector<LayerLoader *> loaders;

protected:
	World *world;
	ResourceBag *resourceBag;
	WorldStorage *storage;
	
public:
	LayerBuilder(World *_world, ResourceBag *_resourceBag, WorldStorage *_storage);

	void registerLoader(LayerLoader *loader);

	Layer *buildLayer(const std::string &file, const std::string &mimeType);
};

#endif /* __XGISLIB_LAYERBUILDER_H__ */
