#include "config.h"

#include <xgislib/xputils.h>
#include <xgislib/layerset.h>

#ifdef XGIS_REFCOUNT_DEBUG
DebugObserver *dbgObserverLayer = NULL;
#endif /* XGIS_REFCOUNT_DEBUG */

void Layer::onEnter(class LayerSet *set) {
	std::vector<osg::ref_ptr<LayerEventHandlers> >::iterator i =
		eventHandlers.begin();
	while(i != eventHandlers.end()) {
		(*i)->onEnter(set, this);
		i++;
	}
}

void Layer::onLeave(class LayerSet *set) {
	std::vector<osg::ref_ptr<LayerEventHandlers> >::iterator i =
		eventHandlers.begin();
	while(i != eventHandlers.end()) {
		(*i)->onLeave(set, this);
		i++;
	}		
}

Layer::Layer() : osg::Referenced() {
#ifdef XGIS_REFCOUNT_DEBUG
	DebugObserver::createObserver(&dbgObserverLayer);
	dbgObserverLayer->objectCreated();
	addObserver(dbgObserverLayer);
#endif /* XGIS_REFCOUNT_DEBUG */
}

Layer::Layer(osg::Node *_node) : osg::Referenced(), node(_node) {
#ifdef XGIS_REFCOUNT_DEBUG
	DebugObserver::createObserver(&dbgObserverLayer);
	dbgObserverLayer->objectCreated();
	addObserver(dbgObserverLayer);
#endif /* XGIS_REFCOUNT_DEBUG */
}

Layer::~Layer() {
	// XGIS_LOG_INFO("Deleting layer.");	
}

bool LayerSet::hasLayer(const std::string &name) {
	std::vector<osg::ref_ptr<Layer> >::iterator i;
	for(i = layers.begin(); i != layers.end(); i++) {
		if((*i)->getName() == name) {
			return true;
		}
	}
	return false;
}

std::vector<osg::ref_ptr<Layer> >::iterator LayerSet::getLayerIterator() {
	return layers.begin();
}

Layer *LayerSet::getLayer(const std::string &name) {
	std::vector<osg::ref_ptr<Layer> >::iterator i;
	for(i = layers.begin(); i != layers.end(); i++) {
		if((*i)->getName() == name) {
			return (*i).get();
		}
	}
	return NULL;
}

bool LayerSet::addLayer(Layer *layer) {
	if(!hasLayer(layer->getName())) {
		layers.push_back(layer);
		if(NULL != layer->getNode()) {
			layerGroup->addChild(layer->getNode());
		}
		return true;
	}
	return false;
}

bool LayerSet::removeLayer(const std::string &name) {
	std::vector<osg::ref_ptr<Layer> >::iterator i;
	for(i = layers.begin(); i != layers.end(); i++) {
		if((*i)->getName() == name) {
			if(NULL != (*i)->getNode()) {
				layerGroup->removeChild((*i)->getNode());
			}
			(*i) = NULL;
			layers.erase(i);
			return true;
		}
	}
	return false;	
}

void LayerSet::removeAllLayers() {
	std::vector<osg::ref_ptr<Layer> >::iterator i;
	for(i = layers.begin(); i != layers.end(); i++) {
		if(NULL != (*i)->getNode()) {
			layerGroup->removeChild((*i)->getNode());
		}
		(*i) = NULL;
	}
	layers.clear();
}

void LayerSet::onEnter() {
	std::vector<osg::ref_ptr<Layer> >::iterator i;
	for(i = layers.begin(); i != layers.end(); i++) {
		(*i)->onEnter(this);
	}
}

void LayerSet::onLeave() {
	std::vector<osg::ref_ptr<Layer> >::iterator i;
	for(i = layers.begin(); i != layers.end(); i++) {
		(*i)->onLeave(this);
	}	
}
