#ifndef __XGISLIB_LAYERSET_H__
#define __XGISLIB_LAYERSET_H__

#include <string>
#include <vector>

#include <osg/Node>
#include <osg/Group>

#ifdef XGIS_REFCOUNT_DEBUG
#	include <xgislib/debugobserver.h>

extern DebugObserver *dbgObserverLayer;
#endif /* XGIS_REFCOUNT_DEBUG */

class LayerEventHandlers : public osg::Referenced {
protected:
	~LayerEventHandlers() {}
	
public:
	virtual void onEnter(class LayerSet *set, class Layer *layer) {}
	virtual void onLeave(class LayerSet *set, class Layer *layer) {}
};

class Layer : public osg::Referenced {
	std::string name;
	osg::ref_ptr<osg::Node> node;
	std::vector<osg::ref_ptr<LayerEventHandlers> > eventHandlers;

protected:
	~Layer();
	
public:
	Layer();
	Layer(osg::Node *_node);
	
	void setName(const std::string &_name) {
		name = _name;
	}
	
	const std::string &getName() {
		return name;
	}
	
	void setNode(osg::Node *_node) {
		node = _node;
	}
	
	osg::Node *getNode() {
		return node.get();
	}
	
	void addEventHandlers(LayerEventHandlers *handlers) {
		eventHandlers.push_back(handlers);
	}
	
	virtual void onEnter(class LayerSet *set);
	virtual void onLeave(class LayerSet *set);
};

class LayerSet {
	// TODO: should replace with a hashtable for O(1) lookups
	std::vector<osg::ref_ptr<Layer> > layers;
	osg::ref_ptr<osg::Group> layerGroup;

public:
	LayerSet() {
		layerGroup = new osg::Group;
	}
	
	~LayerSet() {
		removeAllLayers();
	}
	
	bool hasLayer(const std::string &name);
	bool hasLayers() {
		return !layers.empty();
	}
	std::vector<osg::ref_ptr<Layer> >::iterator getLayerIterator();
	
	Layer *getLayer(const std::string &name);
	virtual bool addLayer(Layer *layer);
	virtual bool removeLayer(const std::string &name);
	
	void removeAllLayers();
	
	osg::Group *getLayerGroup() { return layerGroup.get(); }

	virtual void onEnter();
	virtual void onLeave();
};

#endif /* __XGISLIB_LAYERSET_H__ */
