#ifndef __SOCALIB_OSGUTILS_H__
#define __SOCALIB_OSGUTILS_H__

#include <osg/Drawable>
#include <osg/Node>
#include <osg/Geode>
#include <osg/Billboard>
#include <osgText/Text>

#include <xgislib/world.h>

class Orbit : public osg::NodeCallback {
public:
   Orbit();
	 virtual void operator()(osg::Node* node, osg::NodeVisitor* nv);

   osg::Matrix getWCMatrix(){return worldCoordMatrix;} 

private:
   osg::Matrix worldCoordMatrix;
   float heading;
};

extern osg::Drawable *osgFindNamedDrawable(const std::string& searchName,
	osg::Node* currNode);

extern osg::Vec3 osgGetVec3FromHeightField(const osg::HeightField *hf,
	int col, int row);

extern osg::Node *osgCreateTransparentSurface(const osg::HeightField *hf,
	bool relative, const osg::Vec4 &baseColour, float depthNonTransparent, World *world);

extern bool osgTestRectIntersection(osg::Vec4 &b1, osg::Vec4 &b2);

/*
*	Because of a bug in osg freetype plug-in,
*	if we need to call both setFont() and setText() on a Text node
*	(which we actually need for all Text nodes),
*	AT LEAST ONE OF THESE CALLS should be in the rendering thread
*	(see http://lists.openscenegraph.org/pipermail/osg-users-openscenegraph.org/2007-August/001606.html)
*	
*	So loader threads should always use DeferredText instead
*	(and MUST NOT call setText, but can do all other operations)
*	DeferredText calls setText automatically on the first update()
*/
class DeferredText : public osgText::Text {

friend class DeferredTextUpdateCallback;

private:
	char *str;
	osgText::String::Encoding encoding;

public:
	DeferredText(const char* _str, osgText::String::Encoding _encoding);
	~DeferredText();
};

/*
*	Text on semi-transparent background (experimental implementation)
*	
*	Contains a DeferredText and a semi-transparent rectangle of the same size,
*	both placed onto a Billboard so as to face the viewer.
*
*	TODO:
*		-fix OSG's Billboard to stay parallel to screen (as opposed to facing the eye)
*			(requires messing with OSG because Billboard::computeMatrix	is not virtual)
*		-add upper screen size limit to OSG's Billboard (see above)
*		-make prettier
*		-add parameters for color, transparency etc
*		-merge with DeferredText and make background optional
*/

class TextPanel : public osg::Billboard {
friend class TextPanelUpdateCallback;

private:
	DeferredText *text;
	osg::Geometry *createXZRect(const osg::Vec3 &pos, float w, float h, float alpha);
	float border;

public:
	TextPanel(const char *_str, osgText::String::Encoding _encoding, float _border = 0.0f);
	
	void setFont(osgText::Font *font = 0);
	DeferredText *getText() { return text; }
};

#endif /* __SOCALIB_OSGUTILS_H__ */
