#ifndef __OBSCENE_QUADSINGLE_H__
#  define __OBSCENE_QUADSINGLE_H__ 

#include <osg/Group>

#include <xgislib/coords.h>
#include <xgislib/storage.h>

#ifdef XGIS_REFCOUNT_DEBUG
#	include <xgislib/debugobserver.h>

extern DebugObserver *dbgObserverQuadSingle;
#endif /* XGIS_REFCOUNT_DEBUG */


class QuadSingle : public osg::Group
{
protected:
	static const std::string terrainDescription;
	
	Coords::GaussKr base;     // LL-corner
	int quadResol, texResol;  // resolutions in points-per-km
	int size;                 // size in m
	float dx;                 // interval between quad points in m
	                          // (dy is the same as dx)

	virtual ~QuadSingle();
	
public:
	QuadSingle(const Coords::GaussKr &_base, int quadSize,
		const osg::HeightField *cmodel, const osg::Image *ctexture);
	QuadSingle(const Coords::GaussKr &_base,
		int _size, int _quadResol, int _texResol,
		osg::HeightField *heightField, osg::Image *image);
	
	int getQuadResolution() { return quadResol; }
	int getTextureResolution() { return texResol; }
	
	static osg::Geode *compileTerrain
		(osg::HeightField *heightField, osg::Image *image,
		int quadSize, int quadResol, int texResol,
		const Coords::GaussKr &base);
	
	static const std::string &getTerrainGeodeDescription() {
		return terrainDescription;
	}
};

#endif /* __OBSCENE_QUADSINGLE_H__ */
