#ifndef __XGISLIB_READERWRITERGRD6_H__
#  define __XGISLIB_READERWRITERGRD6_H__

#include <osgDB/FileNameUtils>
#include <osgDB/FileUtils>

/****************************************************************************
 *
 * OpenSceneGraph Reader/Writer plugin for "Surfer 6 Grid" format
 *
 * Written by Jaka Mocnik and Marjan Sterk, XLAB d.o.o., 2008
 *
 **********************************************************************/

class ReaderWriterGrd6 : public osgDB::ReaderWriter
{
public:
    virtual const char* className() const { return "Surfer 6 GRD Reader/Writer"; }

    virtual bool acceptsExtension(const std::string& extension) const
    {
       return osgDB::equalCaseInsensitive(extension, "grd");
    }

    ReadResult readGrd6Stream(std::istream &fin) const;
	WriteResult writeGrd6Stream(std::ostream &fout, const osg::HeightField &hf) const;

	virtual ReadResult readHeightField(std::istream& fin,const osgDB::ReaderWriter::Options* =NULL) const
    {
      return readGrd6Stream(fin);
    }

	virtual ReadResult readHeightField(const std::string& file, const osgDB::ReaderWriter::Options* options) const
	{
		std::string ext = osgDB::getLowerCaseFileExtension(file);
		if (!acceptsExtension(ext))
			return ReadResult::FILE_NOT_HANDLED;

		std::string fileName = osgDB::findDataFile( file, options );
		if (fileName.empty())
			return ReadResult::FILE_NOT_FOUND;

		std::ifstream istream(fileName.c_str(), std::ios::in | std::ios::binary);
		if(!istream)
			return ReadResult::FILE_NOT_HANDLED;

		return readGrd6Stream(istream);
	}
    
	virtual WriteResult writeHeightField(const osg::HeightField& hf,
		std::ostream& fout, const osgDB::ReaderWriter::Options *options) const
	{
		WriteResult ws = writeGrd6Stream(fout, hf);
		return ws;
	}

	virtual WriteResult writeHeightField(const osg::HeightField &hf,
		const std::string& fileName,
		const osgDB::ReaderWriter::Options *options) const
	{
		std::string ext = osgDB::getFileExtension(fileName);
		if(!acceptsExtension(ext))
			return WriteResult::FILE_NOT_HANDLED;

		std::ofstream fout(fileName.c_str(), std::ios::out | std::ios::binary);
		if(!fout)
			return WriteResult::ERROR_IN_WRITING_FILE;

		return writeGrd6Stream(fout, hf);
	}
    
};

#endif /* __XGISLIB_READERWRITERGRD6_H__ */
