#include "config.h"

#include "readerwriterqcol.h"
#include "readerwriterquad.h"

#include "xgislib/xputils.h"
#include <assert.h>

#include <osg/Image>

/****************************************************************************
 *
 * code based on the osg jpeg plugin 
 *
 **********************************************************************/

ReaderWriterQcol::ReadResult
ReaderWriterQcol::readQcolStream(std::istream& fin) const {
	//process header
	QuadHeader header;
	fin.read( (char*)(&header), sizeof(header) / sizeof(char) );
	header.assertValid();
	
	unsigned int
		colCount = header.size*header.resol/1000 + 1,
		rowCount = colCount;

	assert(sizeof(char) == 1); //we assume char is 8 bits
	unsigned char *data = new unsigned char[4 * colCount * rowCount];
	fin.read( (char*)data, 4 * colCount * rowCount );
	if (fin.rdstate() & std::ios::failbit) {
		osg::notify(osg::WARN);
		XGIS_LOG_ERROR("Error: quad file size incorrect!\n");
		return ReadResult::FILE_NOT_HANDLED;
	}

	osg::Image *im = new osg::Image();
	im->setOrigin(osg::Image::BOTTOM_LEFT);
	im->setImage(colCount, rowCount, 1, GL_RGBA8, GL_RGBA,
		GL_UNSIGNED_BYTE, data, osg::Image::USE_NEW_DELETE);
	return im;
}

osg::Image* ReaderWriterQcol::readSimple(const char* filename) {
	QuadHeader header;
	FILE *f = fopen(filename, "rb");

	if (1 != fread( &header, sizeof(header), 1, f )) {
		XGIS_LOG_ERROR("Error reading header");
		return NULL;
	}
	header.assertValid();

	int
		colCount = header.size*header.resol/1000 + 1,
		rowCount = colCount,
		toRead = 4 * colCount * rowCount;

	assert(sizeof(char) == 1); //we assume char is 8 bits
	unsigned char *data = new unsigned char[toRead];
	int actuallyRead = fread( data, 1, 4 * colCount * rowCount, f );
	if (actuallyRead != toRead) {
		osg::notify(osg::WARN);
		XGIS_LOG_ERROR("Error: quad file size incorrect!\n");
		return NULL;
	}
	fclose(f);

	osg::Image *im = new osg::Image();
	im->setOrigin(osg::Image::BOTTOM_LEFT);
	im->setImage(colCount, rowCount, 1, GL_RGBA8, GL_RGBA,
		GL_UNSIGNED_BYTE, data, osg::Image::USE_NEW_DELETE);
	return im;
}
