#ifndef __SOCALIB_READERWRITERQCOL_H__
#  define __SOCALIB_READERWRITERQCOL_H__

#include <osgDB/FileNameUtils>
#include <osgDB/FileUtils>

/****************************************************************************
 *
 * code based on the osg jpeg plugin 
 *
 **********************************************************************/

class ReaderWriterQcol : public osgDB::ReaderWriter
{
	public:
    virtual const char* className() const { return "Qcol Binary Data Reader/Writer"; }

    virtual bool acceptsExtension(const std::string& extension) const
    {
       return osgDB::equalCaseInsensitive(extension, "qcol");
    }

    ReadResult readQcolStream(std::istream &fin) const;

	virtual ReadResult readImage(std::istream& fin,const osgDB::ReaderWriter::Options* =NULL) const
    {
      return readQcolStream(fin);
    }

	virtual ReadResult readImage(const std::string& file, const osgDB::ReaderWriter::Options* options) const
    {
		std::string ext = osgDB::getLowerCaseFileExtension(file);
		if (!acceptsExtension(ext))
			return ReadResult::FILE_NOT_HANDLED;

		std::string fileName = osgDB::findDataFile( file, options );
		if (fileName.empty())
			return ReadResult::FILE_NOT_FOUND;

		std::ifstream istream(fileName.c_str(), std::ios::in | std::ios::binary);
		if(!istream)
			return ReadResult::FILE_NOT_HANDLED;

		ReadResult rr = readQcolStream(istream);
		if(rr.validImage())
			rr.getImage()->setFileName(file);

		return rr;
    }

	static osg::Image* readSimple(const char* filename);
};

#endif /* __SOCALIB_READERWRITERQCOL_H__ */
