#include "config.h"

#include "readerwriterquad.h"

#include "xgislib/xputils.h"

/****************************************************************************
 *
 * code based on the osg jpeg plugin 
 *
 **********************************************************************/

ReaderWriterQuad::ReadResult
ReaderWriterQuad::readQuadStream(std::istream& fin) const {
	//process header
	QuadHeader header;
	fin.read( (char*)(&header), sizeof(header) / sizeof(char) );
	header.assertValid();
	
	osg::HeightField *hf = new osg::HeightField();
	unsigned int
		colCount = header.size*header.resol/1000 + 1,
		rowCount = colCount;
	hf->allocate(colCount, rowCount);
	hf->setOrigin(osg::Vec3(header.xLL, header.yLL, 0));
	hf->setXInterval(1000.0 / header.resol);
	hf->setYInterval(1000.0 / header.resol);

	float *data = new float[colCount * rowCount];
	fin.read( (char*)data, colCount * rowCount * sizeof(float) / sizeof(char) );
	if (fin.rdstate() & std::ios::failbit) {
		osg::notify(osg::WARN);
		XGIS_LOG_ERROR("Error: quad file size incorrect!\n");
		return ReadResult::FILE_NOT_HANDLED;
	}

	float *p = data;
	for (unsigned int col = 0; col < colCount; col++)
		for (unsigned int row = 0; row < rowCount; row++) {
			hf->setHeight(col, row, *p);
			p++;
		}

	delete[] data;
	return hf;
}

osg::HeightField* ReaderWriterQuad::readSimple(const char* filename) {
	QuadHeader header;
	FILE *f = fopen(filename, "rb");

	if (1 != fread( &header, sizeof(header), 1, f )) {
		XGIS_LOG_ERROR("Error reading header");
		return NULL;
	}
	header.assertValid();
	
	osg::HeightField *hf = new osg::HeightField();
	int
		colCount = header.size*header.resol/1000 + 1,
		rowCount = colCount,
		toRead = colCount * rowCount;

	hf->allocate(colCount, rowCount);
	hf->setOrigin(osg::Vec3(header.xLL, header.yLL, 0));
	hf->setXInterval(1000.0 / header.resol);
	hf->setYInterval(1000.0 / header.resol);

	float *data = new float[colCount * rowCount];
	int actuallyRead = fread( data, sizeof(float), toRead, f );
	if (actuallyRead != toRead) {
		osg::notify(osg::WARN);
		XGIS_LOG_ERROR("Error: quad file size incorrect!\n");
		return NULL;
	}
	fclose(f);

	float *p = data;
	for (int col = 0; col < colCount; col++)
		for (int row = 0; row < rowCount; row++) {
			hf->setHeight(col, row, *p);
			p++;
		}

	delete[] data;
	return hf;
}

ReaderWriterQuad::WriteResult
ReaderWriterQuad::writeQuadStream(std::ostream &fout, const osg::HeightField &hf) const {
	assert("ReaderWriterQuad::writeQuadStream() IS OUTDATED - IT ONLY WORKS FOR HEADERLESS QUADS" == NULL);
	size_t s = hf.getNumRows();
	for(size_t i = 0; i < s; i++) {
		for(size_t j = 0; j < s; j++) {
			float h = hf.getHeight(j, i);
			fout.put(*((char *)(&h) + 0));
			fout.put(*((char *)(&h) + 1));
			fout.put(*((char *)(&h) + 2));
			fout.put(*((char *)(&h) + 3));
			if(fout.fail())
				return WriteResult::ERROR_IN_WRITING_FILE;
		}
	}
	
	return WriteResult::FILE_SAVED;
}
