#ifndef __SOCALIB_READERWRITERQUAD_H__
#  define __SOCALIB_READERWRITERQUAD_H__

#include <osgDB/FileNameUtils>
#include <osgDB/FileUtils>

#include <assert.h>
/****************************************************************************
 *
 * code based on the osg jpeg plugin 
 *
 **********************************************************************/

struct QuadHeader {
	int size; //in metres
	int resol; //in points/km
	int xLL, yLL; //GaussKrueger coordinates of LL corner

	void assertValid() {
		assert(0 < size && size < 10*1000*1000); //may detect any Endian-ness issues
		assert(0 < resol);
		assert(size*resol % 1000 == 0);
	}
};

class ReaderWriterQuad : public osgDB::ReaderWriter
{
	public:
    virtual const char* className() const { return "Quad Binary Data Reader/Writer"; }

    virtual bool acceptsExtension(const std::string& extension) const
    {
       return osgDB::equalCaseInsensitive(extension, "quad");
    }

    ReadResult readQuadStream(std::istream &fin) const;
	WriteResult writeQuadStream(std::ostream &fout, const osg::HeightField &hf) const;

	virtual ReadResult readHeightField(std::istream& fin,const osgDB::ReaderWriter::Options* =NULL) const
    {
      return readQuadStream(fin);
    }

	virtual ReadResult readHeightField(const std::string& file, const osgDB::ReaderWriter::Options* options) const
	{
		std::string ext = osgDB::getLowerCaseFileExtension(file);
		if (!acceptsExtension(ext))
			return ReadResult::FILE_NOT_HANDLED;

		std::string fileName = osgDB::findDataFile( file, options );
		if (fileName.empty())
			return ReadResult::FILE_NOT_FOUND;

		std::ifstream istream(fileName.c_str(), std::ios::in | std::ios::binary);
		if(!istream)
			return ReadResult::FILE_NOT_HANDLED;

		ReadResult rr = readQuadStream(istream);
		if(rr.validImage())
			rr.getImage()->setFileName(file);

		return rr;
	}
    
	static osg::HeightField* readSimple(const char* filename);

	virtual WriteResult writeHeightField(const osg::HeightField& hf,
		std::ostream& fout, const osgDB::ReaderWriter::Options *options) const
	{
		WriteResult ws = writeQuadStream(fout, hf);
		return ws;
	}

	virtual WriteResult writeHeightField(const osg::HeightField &hf,
		const std::string& fileName,
		const osgDB::ReaderWriter::Options *options) const
	{
		std::string ext = osgDB::getFileExtension(fileName);
		if(!acceptsExtension(ext))
			return WriteResult::FILE_NOT_HANDLED;

		std::ofstream fout(fileName.c_str(), std::ios::out | std::ios::binary);
		if(!fout)
			return WriteResult::ERROR_IN_WRITING_FILE;

		return writeQuadStream(fout, hf);
	}
    
};

#endif /* __SOCALIB_READERWRITERQUAD_H__ */
