#include <config.h>
#include <stdlib.h>
#include <xgislib/relativealtitudegeometry.h>

class RAGTerrainChangeHandler : public TerrainChangeHandler {
protected:
	RelativeAltitudeGeometry *rag;

public:
	RAGTerrainChangeHandler(RelativeAltitudeGeometry *_rag) {
		rag = _rag;
	}

	virtual osg::Vec4 getTerrainBound() {
		osg::BoundingSphere b = rag->getBound();
		float r = b.radius();
		return osg::Vec4f(b.center().x() - r, b.center().y() - r, 2*r, 2*r);
	}

	virtual bool handle(const osg::Vec4f &bound) {
		rag->recalculateHeights();
		return true;
	}

};

RelativeAltitudeGeometry::RelativeAltitudeGeometry(const RelativeAltitudeGeometry& geometry,
												   const osg::CopyOp& copyop):
												   Geometry(geometry, copyop),
													originalHeights(geometry.originalHeights),
													worldReference(geometry.worldReference)

{
}

RelativeAltitudeGeometry::~RelativeAltitudeGeometry() {
	if(worldReference.valid()) {
		XGIS_LOG_DEBUG5("Removing terrain change handler...");
		worldReference->removeTerrainChangeHandler(heightChangeHandler);
	}
}

void RelativeAltitudeGeometry::saveOriginalState(World* world)
{
	worldReference = world;
	//printf("RelativeAltitudeGeometry::saveOriginalState::Enter\n");

	osg::Vec3Array* vertices = dynamic_cast<osg::Vec3Array*>(getVertexArray());

	if(vertices==NULL)
		return;

	//originalHeights = new osg::Vec3Array();
	for(unsigned int i=0; i<vertices->getNumElements(); i++)
		originalHeights.push_back( vertices->operator[](i).z() );
	//printf("RelativeAltitudeGeometry::saveOriginalState::vertices->size()==%u\n", vertices->size());

	heightChangeHandler = new RAGTerrainChangeHandler(this);
	world->addTerrainChangeHandler(heightChangeHandler);
}

void RelativeAltitudeGeometry::recalculateHeights(void)
{

	if(!worldReference.valid()) {
		return;
	}

	osg::Vec3Array* vertices = dynamic_cast<osg::Vec3Array*>(getVertexArray());
	for(unsigned int i=0; i<originalHeights.size(); i++)
	{
		osg::Vec3 v = vertices->operator[](i);

		dynamic_cast<osg::Vec3Array*>(getVertexArray())->operator[](i).set(v.x(), v.y(),
						originalHeights[i] + worldReference->getTerrainHeight( v.x(), v.y() ) +
						0.5f /*+ (rand()/(2.0*RAND_MAX))*/  ); // To avoid Z Buffer fighting we raise everithing .05mts
                                                           // Since 2 layers might overlap, we randomly set diff hights

		// Plus half meter to avoid clipping!

		// FIXME: shouldn't we recalculate the normals as well? this could
		// prove a bit troublesome, though, as normal calculation depends on
		// the type of geometry... JM
	}

	dirtyDisplayList(); dirtyBound();

	return;
}
