#include <config.h>

#include <xgislib/relativealtitudegroup.h>

class RAGroupTerrainChangeHandler : public TerrainChangeHandler {
protected:
	RelativeAltitudeGroup *rag;
	
public:
	RAGroupTerrainChangeHandler(RelativeAltitudeGroup *_rag) {
		rag = _rag;
	}
	
	virtual osg::Vec4 getTerrainBound() {
		osg::BoundingSphere b = rag->getBound();
		float r = b.radius();
		return osg::Vec4f(b.center().x() - r, b.center().y() - r, 2*r, 2*r);
	}
	
	virtual bool handle(const osg::Vec4f &bound) {
		// XGIS_LOG_INFO("Calling terrain change handler...");
		rag->recalculateAltitude();
		return true;
	}

};


RelativeAltitudeGroup::~RelativeAltitudeGroup() {
	if(world.valid()) {
		XGIS_LOG_DEBUG5("Removing terrain change handler...");
		world->removeTerrainChangeHandler
			(heightChangeHandler);
	}
}

RelativeAltitudeGroup::RelativeAltitudeGroup
	(const osg::Vec2f &_refPosition, float _relAltitude, World *_world) :
	PositionAttitudeTransform(), world(_world),
	refPosition(_refPosition), relAltitude(_relAltitude) {

	heightChangeHandler = new RAGroupTerrainChangeHandler(this); 
	world->addTerrainChangeHandler(heightChangeHandler);
	recalculateAltitude();
}
	
void RelativeAltitudeGroup::recalculateAltitude() {
	float z = relAltitude + world->getTerrainHeight(refPosition.x(), refPosition.y());
	setPosition(osg::Vec3(0, 0, z));
}
