#ifndef __OBSCENE_RESOURCEBAG_H__
#define __OBSCENE_RESOURCEBAG_H__

#include <osgText/Font>
#include <osg/Image>
#include <OpenThreads/Mutex>

#include <xgislib/storage.h>

typedef enum {
	FONT_TIMES = 0,
	FONT_LAST = 1
} FontID;

typedef enum {
	ROOF_TEXTURE_1 = 0,
	ROOF_TEXTURE_2 = 1,
	ROOF_TEXTURE_LAST = 2
} RoofTextureID;

typedef enum {
	WALL_TEXTURE_1 = 0,
	WALL_TEXTURE_2 = 1,
	WALL_TEXTURE_3 = 2,
	WALL_TEXTURE_4 = 3,
	WALL_TEXTURE_LAST = 4
} WallTextureID;

class ResourceBag {
	
private:
	static const std::string fontNames[FONT_LAST];
	static const std::string roofTextureNames[ROOF_TEXTURE_LAST];
	static const std::string wallTextureNames[WALL_TEXTURE_LAST];

	osg::ref_ptr<osgText::Font> fonts[FONT_LAST];
	osg::ref_ptr<osg::Image> roofImages[ROOF_TEXTURE_LAST];
	osg::ref_ptr<osg::Image> wallImages[WALL_TEXTURE_LAST];	
	
	// TODO: use a hashtable for the following for O(1) lookups
	std::vector<osg::ref_ptr<osg::Node> > models;
	std::vector<std::string> fetchedTextures;

	OpenThreads::Mutex mutex;

public:
	ResourceBag();

	osgText::Font *getFont(FontID id);
	osg::Node *getModel(const std::string &name,
		const std::vector<std::string> &textures, WorldStorage *storage);
	osg::Image *getRoofTexture(RoofTextureID id, WorldStorage *storage);
	osg::Image *getWallTexture(WallTextureID id, WorldStorage *storage);
};

#endif /* __OBSCENE_RESOURCEBAG_H__ */
