#include <config.h>

#include "stdlib.h"

#include <algorithm>

#include <xgislib/xgislog.h>

#include <xgislib/stratus.h>
#include <xgislib/pointdata.h>
#include <xgislib/xgisimgutils.h>

#include <osg/LOD>
#include <osg/BlendFunc>
#include <osg/PointSprite>
#include <osg/Point>
#include <osgDB/ReadFile>
#include <osg/Texture2D>
#include <osg/PositionAttitudeTransform>

osg::ref_ptr<osg::Image> Stratus::stratusTexture = NULL;
osg::ref_ptr<osg::Geode> Stratus::cloudNear = NULL;

Stratus::Stratus(WorldStorage *_storage, int _numParticles, int _partSize, 
	float _sizeX, float _sizeY, float _thick, const osg::Vec3f &_sunPos, 
	time_t _startTime, float _rangeMin, float _rangeMax) : 
	Cloud(_storage, _numParticles, _partSize, _sizeX, _sizeY, _thick, _sunPos,
		_startTime) {

	this->rangeMin = _rangeMin;
	this->rangeMax = _rangeMax;

	placeParticles();
}

void Stratus::shadeCloud(osg::Image *image, float altitude){
	osg::Vec3f lightColour = getLightColour(altitude);
	int width = image->s();
	int height = image->t();

	for (int i = 0; i < width; i++) {
		for (int j = 0; j < height; j++) {
			osg::Vec4f *data = (osg::Vec4f*)image->data(i,j);
			osg::Vec4f colour = osg::Vec4f(data->r()*lightColour.x(), 
				data->g()*lightColour.y(), 
				data->b()*lightColour.z(), 
				data->a());

			*data = colour;
		}
	}
}

void Stratus::getStratusColour(float altitude, float &colourHi, 
							   float &colourLow) {
	float c1 = 8.0;
	float c2 = -2.0;
	float goal = 0.3;

	osg::Vec4f ret;
	

	if (altitude > c1)  {
		colourHi = this->rangeMax;
		colourLow = this->rangeMin;
	} else {
		float val = this->rangeMax;
		val -= (c1 - altitude)/(c1 - c2) * (val - goal);
		val = MAX(val, goal);

		colourHi = val;
		colourLow = val - 0.1;//(this->rangeMax - this->rangeMin);

		colourLow = MAX(colourLow, 0);
	}

	

}


void Stratus::placeParticles() {


	osg::Geometry *cloud = new osg::Geometry();
	osg::Vec3Array *vertices = new osg::Vec3Array();
	osg::Vec4Array *colours = new osg::Vec4Array();

	colours->push_back(osg::Vec4f(1.0, 1.0, 1.0, 1.0));

	osg::ref_ptr<osg::Geode> cloudGeode = new osg::Geode();



	//osg::ref_ptr<osg::Image> image = new osg::Image;

	int numPixels = 129;
	
	XGIS_LOG_DEBUG2("stratus texture: %p", stratusTexture.get());
	XGIS_LOG_DEBUG2("texture valid: %d", (((stratusTexture.get() != NULL)
		&&(stratusTexture->valid())) ? 1 : 0));

	if ((stratusTexture.get() == NULL) || (NULL == stratusTexture->data()) 
		|| (!stratusTexture->valid())) {

		float rmin, rmax;
		double azimuth, altitude;
		Coords::GaussKr pos(sunPosition.x(), sunPosition.y());

		Pointdata::FeatureInfo::getSunPosition(pos, startTime, azimuth, altitude);

		getStratusColour(altitude, rmax, rmin);
		XGIS_LOG_DEBUG2("stratus range %f %f", rmin, rmax);

		stratusTexture = new osg::Image();
		stratusTexture->allocateImage(numPixels,numPixels,1,GL_RGBA,GL_FLOAT);
		stratusTexture->setInternalTextureFormat(GL_RGBA);

		XGisImage::initImage(stratusTexture.get());
		XGisImage::generateFractal(stratusTexture.get(), 1.0, false, true, false);
		XGisImage::scaleImage(stratusTexture.get(), numPixels-1, numPixels-1);
		XGisImage::setGrayRange(stratusTexture.get(), rmin, rmax, false);

		XGisImage::makeCircle(stratusTexture.get());
		//shadeCloud(stratusTexture.get(), altitude);

		cloudNear = generateCloudFromImage(stratusTexture.get());

	} 


	
	//XGisImage::addPercentNoise(image.get(), 0.04);

	//XGisImage::setGrayRange(image.get(), 0.8, 0.95);
	//XGisImage::convolve(image.get(), 0);
	//XGisImage::smoothBorder(image.get(), 10, true, 0);

#if 0	
	for(int i=0; i<image->s();i++){
		for(int j=0; j<image->t();j++){
			osg::Vec4f *data = (osg::Vec4f*)image->data(i,j);
/*			if (data->r() < 0 ) */XGIS_LOG_DEBUG2("%f %d %d", data->r(), i, j);
		}
	}
#endif


	osg::Texture2D *tex = new osg::Texture2D();

	tex->setImage(stratusTexture.get());
	
	vertices->push_back(osg::Vec3f(0,0,0));
	vertices->push_back(osg::Vec3f(sizeX,0,0));
	vertices->push_back(osg::Vec3f(sizeX,sizeY,0));
	vertices->push_back(osg::Vec3f(0,sizeY,0));

	osg::Vec2Array* texCoords = new osg::Vec2Array(4);
   (*texCoords)[0].set(0.0f,0.0f);
   (*texCoords)[1].set(1.0f,0.0f);
   (*texCoords)[2].set(1.0f,1.0f);
   (*texCoords)[3].set(0.0f,1.0f);
   cloud->setTexCoordArray(0,texCoords);
   osg::StateSet *cloudState = cloud->getOrCreateStateSet();

   tex->setFilter(osg::Texture2D::MIN_FILTER,
	   osg::Texture2D::LINEAR_MIPMAP_LINEAR);
   tex->setFilter(osg::Texture2D::MAG_FILTER,
	   osg::Texture2D::LINEAR_MIPMAP_LINEAR);

	cloudState->setMode(GL_BLEND, osg::StateAttribute::ON);
	osg::BlendFunc *fn = new osg::BlendFunc();
	fn->setFunction(osg::BlendFunc::SRC_ALPHA, osg::BlendFunc::ONE_MINUS_SRC_ALPHA);
	cloudState->setAttributeAndModes(fn, osg::StateAttribute::ON);

	tex->setWrap(osg::Texture2D::WRAP_S, osg::Texture2D::CLAMP_TO_EDGE);
	tex->setWrap(osg::Texture2D::WRAP_T, osg::Texture2D::CLAMP_TO_EDGE);
   
   cloudState->setTextureAttribute(0,tex,osg::StateAttribute::ON);
	cloudState->setMode(GL_LIGHTING, osg::StateAttribute::OFF);
	cloudState->setMode(GL_DEPTH_TEST, osg::StateAttribute::ON);   
	cloudState->setRenderBinDetails(8,"RenderBin");

   cloudState->setTextureMode(0, GL_TEXTURE_2D, osg::StateAttribute::ON );
	cloud->setVertexArray(vertices);
	cloud->setColorArray(colours);
	cloud->setColorBinding(osg::Geometry::BIND_OVERALL);
	cloud->addPrimitiveSet(new osg::DrawArrays(osg::PrimitiveSet::QUADS, 0,4));

	cloudGeode->addDrawable(cloud);

	
	
	osg::ref_ptr<osg::LOD> cloudLOD = new osg::LOD();
	
	cloudLOD->addChild(cloudGeode.get(),2001.0,1e10);
	cloudLOD->addChild(cloudNear.get(),0,2000.0);

	this->addChild(cloudLOD.get());

}

osg::Geode *Stratus::generateCloudFromImage(osg::Image *cloudMap) {

	
	int width = cloudMap->s();
	int height = cloudMap->t();

	osg::Geometry *cloud = new osg::Geometry();
	osg::Vec3Array *vertices = new osg::Vec3Array();
	osg::Vec4Array *colours = new osg::Vec4Array();
	osg::Geode *cloudGeode = new osg::Geode();

/*set state to particly cloud*/
	osg::StateSet *cloudState = cloud->getOrCreateStateSet();

	osg::PointSprite *sprite = new osg::PointSprite();
	cloudState->setTextureAttributeAndModes(0, sprite, osg::StateAttribute::ON);

	osg::Point *point = new osg::Point();
	point->setSize((float)(particleSize));

	cloudState->setAttribute(point);

	cloudState->setMode(GL_BLEND, osg::StateAttribute::ON);
	osg::BlendFunc *fn = new osg::BlendFunc();
	fn->setFunction(osg::BlendFunc::SRC_ALPHA, osg::BlendFunc::ONE_MINUS_SRC_ALPHA);
	cloudState->setAttributeAndModes(fn, osg::StateAttribute::ON);
	cloudState->setRenderBinDetails(8,"RenderBin");

	/// The texture for the sprites
	osg::Texture2D *tex = new osg::Texture2D();

	osg::Image *image = storage->getImage("smoke.rgb");
	if (image != NULL) {
		tex->setImage(image);
	} else {
		XGIS_LOG_DEBUG2("Cloud particle image not found!");
	}

	cloudState->setTextureAttributeAndModes(0, tex, osg::StateAttribute::ON);


	float unit = (float)particleSize/10.0;

	osg::Vec4f e = osg::Vec4f(1,1,1,0);

	for (float i = 0; i < sizeX; i+= unit) {
		for (float j = 0; j < sizeY; j+= unit) {
			for (float k =0; k < thickness; k+= unit) {

				float offX = (((rand() % 100)/ 100.0) - 0.5)*unit;
				float offY = (((rand() % 100)/ 100.0) - 0.5)*unit;
				float offZ = (((rand() % 100)/ 100.0) - 0.5)*unit*0.5;

				vertices->push_back(osg::Vec3f(i + offX,j + offY,k + offZ));

				int imgX = osg::round((i/(sizeX))*width);
				int imgY = osg::round((j/(sizeY))*height);

				osg::Vec4f colour = XGisImage::getImageValue(cloudMap,imgX, imgY,e);

				colours->push_back(colour);
			}

		}
	}

	cloud->setVertexArray(vertices);
	cloud->setColorArray(colours);
	cloud->setColorBinding(osg::Geometry::BIND_PER_VERTEX);
	cloud->addPrimitiveSet(new osg::DrawArrays(osg::PrimitiveSet::POINTS, 
		0,vertices->size()));
	cloudGeode->addDrawable(cloud);

	XGIS_LOG_DEBUG2("vertices %d", vertices->size());

	return cloudGeode;
}
