#include "config.h"

#include <xgislib/worldloader.h>
#include <xgislib/xgisconfig.h>

#include <xgislib/layerbuilder.h>
#include <xgislib/xputils.h>
#include <xgislib/osgutils.h>

const std::string LoadDetails::taskId = "LoadDetails";

void LoadDetails::load() {
	if(NULL == dl) {
		dl = w->getDetailLevel(q.get());
	}

	if(dl->maxDistance > 1.0) {
		// detail level found. load it. the method handles the
		// case when there is no change.
		if(!q->hasFine(dl->hfResolution,
			dl->txtResolution,
			dl->subdivision)) {
			// try reading texture
			osg::ref_ptr<osg::Image> image =
				getStorage()->getTexture(dl->txtResolution, q->getSize(),
					q->getBase());
			if (!image.valid()) {
				XGIS_LOG_WARNING("Failed loading texture.\n");
				return;
			}
	
			osg::ref_ptr<osg::HeightField> heightField =
				getStorage()->getQuad(dl->hfResolution, q->getSize(),
					q->getBase());
			if (!heightField.valid()) {
				XGIS_LOG_WARNING("Failed loading height field.");			
				return;
			}

			terrain = new QuadSingle(q->getBase(), q->getSize(),
				dl->hfResolution, dl->txtResolution,
				heightField.get(), image.get());
		}
	}
	else if(q->hasFine()) {
		// no detail level found: use coarse model if not already set
		clearFine = true;
	}
}

void LoadDetails::updateWorld() {
	if(clearFine) {
		q->clearFine();
	}
	else if(terrain.valid()) {
		q->setFine(terrain.get());
	}
}

const std::string LoadLayers::taskId = "LoadLayers";

void LoadLayers::load() {
	DetailLevel *d = w->getDetailLevel(q.get());
	if(d->layers) {
		std::vector<std::string>::const_iterator i =
			layersToLoad.begin();
		std::vector<std::string>::const_iterator g =
			layersToLoad.end();
		while(i != g) {
			if(!q->hasLayer(*i)) {
				std::string layerFilePath =
					getStorage()->getLayer(*i, q->getSize(), q->getBase());
				// TODO: currently, we can only load per-quad xgf files
				// the whole layer infrastructure should be remade from the
				// ground up so that layers are *never* bound per-quad, but
				// are retrieved per-bounding-box
				Layer *l = w->getLayerBuilder()->buildLayer(layerFilePath,
					"application/vnd.xlab-gis.xgf+xml");
				if(NULL == l) {
					/* loading faled; add empty layer to prevent a retry */
					l = new Layer();
				}
				l->setName(*i);
				layers.push_back(l);
			}
			i++;
		}
	}
}

void LoadLayers::updateWorld() {
	if(q->hasFine()) {
		std::vector<osg::ref_ptr<Layer> >::const_iterator i = layers.begin();
		std::vector<osg::ref_ptr<Layer> >::const_iterator g = layers.end();
		while(i != g) {
			if(!q->hasLayer((*i)->getName()) && q->addLayer((*i).get())) {
				/* added */
			} else {
				/* freeing of Layer should be performed by osg refcounting */
			}
			i++;
		}
	}
	else {
		/* freeing of Layers should be performed by osg refcounting */
	}
}
