#ifndef __OBSCENE_WORLDLOADER_H__
#	define __OBSCENE_WORLDLOADER_H__

#include <vector>

#include <xgislib/coords.h>
#include <xgislib/quad.h>
#include <xgislib/quadsingle.h>
#include <xgislib/storage.h>
#include <xgislib/layerset.h>

#include <xgislib/world.h>

class LoadDetails : public LoaderTask {
	static const std::string taskId;
	
	osg::ref_ptr<Quad> q;
	osg::ref_ptr<World> w;
	osg::ref_ptr<QuadSingle> terrain;
	bool clearFine;
	DetailLevel *dl;
	DataLoaderTaskPriority pri;
	float dist;

public:
	LoadDetails(Quad *_q, World *_w, DetailLevel *_dl, DataLoaderTaskPriority _pri, float _dist) :
		q(_q), w(_w), dl(_dl), pri(_pri), dist(_dist) {
		terrain = NULL;
		clearFine = false;
		q->addLoaderTask(getTaskId());
	}
	virtual ~LoadDetails() {
		q->removeLoaderTask(getTaskId());
	}
	
	virtual void load();
	virtual void updateWorld();

	virtual int staticPriority() { return pri; }
	virtual int dynamicPriority() { return dist; }

	static const std::string &getTaskId() { return taskId; }
};

class LoadLayers : public LoaderTask {
	static const std::string taskId;

	osg::ref_ptr<Quad> q;
	osg::ref_ptr<World> w;
	float dist;
	const std::vector<std::string> layersToLoad;

	std::vector<osg::ref_ptr<Layer> > layers;

public:
	LoadLayers(Quad *_q, World *_w, float _dist, const std::vector<std::string> &_layersToLoad) :
		q(_q), w(_w), dist(_dist), layersToLoad(_layersToLoad) {
		q->addLoaderTask(getTaskId());
	}
	virtual ~LoadLayers() {
		q->removeLoaderTask(getTaskId());
	}

	virtual void load();
	virtual void updateWorld();

	virtual int staticPriority() { return PRIORITY_LAYERS; }
	virtual int dynamicPriority() { return dist; }

	static const std::string &getTaskId() { return taskId; }
};

#endif /* __OSBCENE_WORLDLOADER_H__ */
