#ifndef __OBSCENE_WORLDUPDATECALLBACK_H__
#	define __OBSCENE_WORLDUPDATECALLBACK_H__

#include <osg/Node>
#include <osg/PolygonMode>
#include <osg/Timer>

#include <xgislib/quad.h>
#include <xgislib/pickaction.h>

class WorldUpdateHandler {
public:
	virtual void handle(class World *world, const osg::FrameStamp *stamp) = 0;	
};

class WorldUpdateCallback : public osg::NodeCallback {
	friend class World;
	
protected:
	bool toggleDetailsFlag;

	osg::PolygonMode *newPolyMode;
	
	std::vector<std::string> layersToRemove, layersToAdd;
	
	std::vector<WorldUpdateHandler *> updateHandlers;
	void fireUpdateEvent(class World *world,
		const std::vector<WorldUpdateHandler *> &handlerChain,
		const osg::FrameStamp *stamp);

	const char *statusText;
	unsigned int statusTimeout;
	
	// adaptive (wrt. processing time) update processing
	osg::Timer updateTimer;
	float updateMaxTime;
	
public:
	WorldUpdateCallback();
	
	virtual void operator()(osg::Node *node, osg::NodeVisitor *nv);

	void setPolygonMode(osg::PolygonMode::Face face,
		osg::PolygonMode::Mode mode) {
		newPolyMode = new osg::PolygonMode(face, mode);		
	}
	
	void toggleDetails() {
		toggleDetailsFlag = true;
	}

	void triggerLayerAdd(const std::string &layer);
	void triggerLayerRemove(const std::string &layer);

	void addUpdateHandler(WorldUpdateHandler *handler) {
		updateHandlers.push_back(handler);
	}

	void setStatus(const char *_statusText, unsigned int _statusTimeout = 5) {
		statusText = _statusText;
		statusTimeout = _statusTimeout;
	}
};

#endif /* __OBSCENE_WORLDUPDATECALLBACK_H__ */
