#include "config.h"

#include <xgislib/xputils.h>
#include <xgislib/xgisconfig.h>

XGisConfig *XGisConfig::instance = NULL;

void XGisConfig::load() {
	fUseCompiledQuads = false;  // currently, compiled quads are poorly supported

	aLayers.push_back("buildings");
	aLayers.push_back("places");
	aLayers.push_back("weather");
	aLayers.push_back("restaurants");
	aLayers.push_back("models");
	aLayers.push_back("surface-flood-mali-graben-1d-200-10");
	aLayers.push_back("vectors-flood-mali-graben-2d");

	floodPath = ".";

	farPlane = 50000.0;
	fogSize = 0.0;
	defaultDetailLevelSet = 0;
}

bool XGisConfig::addLayer(const std::string &layer) {
	std::vector<std::string>::iterator i = vLayers.begin();
	while(i != vLayers.end()) {
		if((*i) == layer) {
			return false;
		}
		i++;
	}
	vLayers.push_back(layer);
	fireAddLayerChange(layer);
	return true;
}

bool XGisConfig::removeLayer(const std::string &layer) {
	std::vector<std::string>::iterator i = vLayers.begin();
	while(i != vLayers.end()) {
		if((*i) == layer) {
			vLayers.erase(i);
			fireRemoveLayerChange(layer);
			return true;
		}
		i++;
	}
	return false;
}

void XGisConfig::fireAddLayerChange(const std::string &layer) {
	std::vector<osg::ref_ptr<XGisConfigChangeHandlers> >::iterator i = changeHandlers.begin();
	while(i != changeHandlers.end()) {
		(*i)->onAddLayer(layer);
		i++;
	}
}

void XGisConfig::fireRemoveLayerChange(const std::string &layer) {
	std::vector<osg::ref_ptr<XGisConfigChangeHandlers> >::iterator i = changeHandlers.begin();
	while(i != changeHandlers.end()) {
		(*i)->onRemoveLayer(layer);
		i++;
	}
}
