#ifndef __OBSCENE_XGISCONFIG_H__
#	define __OBSCENE_XGISCONFIG_H__

#include <osg/Referenced>
#include <osg/Object>

#include <vector>
#include <string>

#include <time.h>

#include <xgislib/gpstrack.h>

class XGisConfigChangeHandlers : public osg::Referenced {
public:
	virtual void onAddLayer(const std::string &layer) {}
	virtual void onRemoveLayer(const std::string &layer) {}
};

class XGisConfig {
private:
	static XGisConfig *instance;
	
	// prevent instantiation
	XGisConfig() {};
	
	// do we use precompiled or raw quad files
	bool fUseCompiledQuads;
	// visible and available layers
	std::vector<std::string> vLayers, aLayers;
	// default rotation and translation speed (for flights)
	float rotSpeed, trsSpeed;
	// far plane + fog size
	float farPlane, fogSize;
	// default detail level set
	int defaultDetailLevelSet;
	// default start time
	time_t startTime;
	// start track
	osg::ref_ptr<GpsTrack> startTrack;
	// path to flood surfaces
	std::string floodPath;
	
	std::vector<osg::ref_ptr<XGisConfigChangeHandlers> > changeHandlers;

	void fireAddLayerChange(const std::string &layer);
	void fireRemoveLayerChange(const std::string &layer);
	
public:
	void load();
	
	bool useCompiledQuads() {
		return fUseCompiledQuads;
	}

	bool addLayer(const std::string &layer);
	bool removeLayer(const std::string &layer);
	const std::vector<std::string> &getVisibleLayers() {
		return vLayers;
	}
	const std::vector<std::string> &getAvailableLayers() {
		return aLayers;
	}

	std::string &getFloodPath() {
		return floodPath;
	}
	void setFloodPath(const std::string &path) {
		floodPath = path;
	}

	void addChangeHandlers(XGisConfigChangeHandlers *handlers) {
		changeHandlers.push_back(handlers);
	}

	float getRotationSpeed() {
		return rotSpeed;
	}
	
	void setRotationSpeed(float _rotSpeed) {
		rotSpeed = _rotSpeed;
	}
	
	float getTranslationSpeed() {
		return trsSpeed;
	}
	
	void setTranslationSpeed(float _trsSpeed) {
		trsSpeed = _trsSpeed;
	}
	
	float getFarPlane() {
		return farPlane;
	}
	
	void setFarPlane(float _farPlane) {
		farPlane = _farPlane;
	}

	float getFogSize() {
		return fogSize;
	}
	
	void setFogSize(float _fogSize) {
		fogSize = _fogSize;
	}

	float getDefaultDetailLevelSet() {
		return defaultDetailLevelSet;
	}
	
	void setDefaultDetailLevelSet(float _defaultDetailLevelSet) {
		defaultDetailLevelSet = _defaultDetailLevelSet;
	}

	time_t getStartTime() {
		return startTime;
	}
	
	void setStartTime(time_t _startTime) {
		startTime = _startTime;
	}
	
	GpsTrack *getStartTrack() {
		return startTrack.get();
	}
	
	void setStartTrack(GpsTrack *_startTrack) {
		startTrack = _startTrack;
	}
	
	static XGisConfig *getInstance() {
		if(NULL == instance) {
			instance = new XGisConfig;
			instance->load();
		}
		return instance;
	}
};

#endif /* !__OBSCENE_XGISCONFIG_H__ */
