/* interface:

	macros to use:
		XGIS_LOG_<level> for each of the levels (see below)

		example:
			XGIS_LOG_DEBUG2("i=%d", i)
			XGIS_LOG_FATAL("There's a fuck-up!");

	parameters:
		XGIS_LOG_FILE:
			define it to a FILE* if you wish to print somewhere else than stderr
		
		XGIS_LOG_USE_TIME:
			define if you want each msg to contain time-stamp
		
		XGIS_LOG_USE_INDENT:
			define to indent less important messages more
		
		XGIS_LOG_USE_FILE_LINE:
			define if you want each msg to contain source file and line number

		verbose level: set by defining one of these
			none - just "fatal" is printed
			XGIS_LOG_LEVEL_ERROR - error and fatal
			XGIS_LOG_LEVEL_WARNING - warning, error and fatal
			XGIS_LOG_LEVEL_INFO
			XGIS_LOG_LEVEL_DEBUG1
			XGIS_LOG_LEVEL_DEBUG2
			XGIS_LOG_LEVEL_DEBUG3
			XGIS_LOG_LEVEL_DEBUG4
			XGIS_LOG_LEVEL_DEBUG5
		
		XGIS_LOG_LEVEL_NO_IMPLICIT_HIGHER_LEVELS:
			if defined, a verbose level DOES NOT imply all higher levels		

	You can change the parameters anywhere in source
	by re-defining them and re-including this file. Example:

	#undef XGIS_LOG_USE_TIME
	#define XGIS_LOG_LEVEL_DEBUG5
	#define XGIS_LOG_REINCLUDE //DON'T FORGET THIS IF YOU ARE RE-INCLUDING!
	#include <xgislib/xgislog.h>

*/

#if defined(XGIS_LOG_REINCLUDE) && defined(__XGISLIB_XGISLOG_H__)
/* undef everything to allow re-including with different parameters */

#	undef XGIS_LOG_INTERNAL_TIME_OFFSET
#	undef XGIS_LOG_INTERNAL_TIME
#	undef XGIS_LOG_INTERNAL_INDENT_UNIT 
#	undef XGIS_LOG_INTERNAL_INDENT
#	undef XGIS_LOG_INTERNAL_MSG_SUBINDENT
#	undef XGIS_LOG_INTERNAL_NUM_DIR_LEVELS
#	undef XGIS_LOG_INTERNAL_MAX_FILENAME_LEN
#	undef XGIS_LOG_INTERNAL_PREFIX
#	undef XGIS_LOG_INTERNAL_SUFFIX

#	undef XGIS_LOG_LEVEL_INTERNAL_DEBUG5
#	undef XGIS_LOG_LEVEL_INTERNAL_DEBUG4
#	undef XGIS_LOG_LEVEL_INTERNAL_DEBUG3
#	undef XGIS_LOG_LEVEL_INTERNAL_DEBUG2
#	undef XGIS_LOG_LEVEL_INTERNAL_DEBUG1
#	undef XGIS_LOG_LEVEL_INTERNAL_INFO
#	undef XGIS_LOG_LEVEL_INTERNAL_WARNING
#	undef XGIS_LOG_LEVEL_INTERNAL_ERROR
#	undef XGIS_LOG_LEVEL_INTERNAL_FATAL

#	undef XGIS_LOG_FATAL
#	undef XGIS_LOG_ERROR
#	undef XGIS_LOG_WARNING
#	undef XGIS_LOG_INFO
#	undef XGIS_LOG_DEBUG1
#	undef XGIS_LOG_DEBUG2
#	undef XGIS_LOG_DEBUG3
#	undef XGIS_LOG_DEBUG4
#	undef XGIS_LOG_DEBUG5

/* now enable the re-including */
#	undef XGIS_LOG_REINCLUDE
#	undef __XGISLIB_XGISLOG_H__

#endif /* XGIS_LOG_REINCLUDE */

#ifndef __XGISLIB_XGISLOG_H__
#define __XGISLIB_XGISLOG_H__


/* if XGIS_LOG_FILE is not defined to something else, we print to stderr */
#ifndef XGIS_LOG_FILE
#	define XGIS_LOG_FILE stderr
#endif /* XGIS_LOG_FILE */

/* macros for use only by other macros */
#ifdef XGIS_LOG_USE_TIME
#	define XGIS_LOG_INTERNAL_TIME do {\
		fprintf(XGIS_LOG_FILE, "at ");\
		xpPrintTimeOfDay(XGIS_LOG_FILE);\
		fprintf(XGIS_LOG_FILE, " ");\
	} while (0)
#else
#	define XGIS_LOG_INTERNAL_TIME do { ; } while(0)
#endif /* XGIS_LOG_USE_TIME */

#ifdef XGIS_LOG_USE_INDENT
#	define XGIS_LOG_INTERNAL_INDENT_UNIT 2
#	define XGIS_LOG_INTERNAL_INDENT(n) do {\
		for (int i = 0; i < (n) * XGIS_LOG_INTERNAL_INDENT_UNIT; i++)\
			fprintf(XGIS_LOG_FILE, " ");\
	} while(0)
#else
#	define XGIS_LOG_INTERNAL_INDENT(n) {}
#endif /* XGIS_LOG_USE_INDENT */

#ifdef XGIS_LOG_USE_FILE_LINE
#define XGIS_LOG_INTERNAL_MSG_SUBINDENT 1
#define XGIS_LOG_INTERNAL_NUM_DIR_LEVELS 1
#define XGIS_LOG_INTERNAL_MAX_FILENAME_LEN 1000
#	define XGIS_LOG_INTERNAL_PREFIX(n, LEVELNAME) do {\
		XGIS_LOG_INTERNAL_INDENT(n);\
		char file[XGIS_LOG_INTERNAL_MAX_FILENAME_LEN+1];\
		strncpy(file, __FILE__, XGIS_LOG_INTERNAL_MAX_FILENAME_LEN);\
		char *buf = file;\
		int numSlashes = 0;\
		while ( NULL != (buf = strchr(buf, dirSlash)) ) {\
			numSlashes++;\
			buf++;\
		}\
		buf = file;\
		while (numSlashes > XGIS_LOG_INTERNAL_NUM_DIR_LEVELS) {\
			buf = strchr(buf, dirSlash) + 1;\
			numSlashes--;\
		}\
		fprintf(XGIS_LOG_FILE, "%s ", (LEVELNAME));\
		XGIS_LOG_INTERNAL_TIME;\
		fprintf(XGIS_LOG_FILE, "in %s, line %d, mgs=\n", buf, __LINE__);\
		XGIS_LOG_INTERNAL_INDENT((n) + (XGIS_LOG_INTERNAL_MSG_SUBINDENT));\
	} while (0)
#else
#	define XGIS_LOG_INTERNAL_PREFIX(n, LEVELNAME) do {\
		XGIS_LOG_INTERNAL_INDENT(n);\
		fprintf(XGIS_LOG_FILE, "%s ", (LEVELNAME));\
		XGIS_LOG_INTERNAL_TIME;\
		fprintf(XGIS_LOG_FILE, ": ");\
	} while (0)
#endif /* XGIS_LOG_FILE_LINE */

#define XGIS_LOG_INTERNAL_SUFFIX(n, LEVELNAME) do {\
	fprintf(XGIS_LOG_FILE, "\n");\
	fflush(XGIS_LOG_FILE);\
} while(0)

/* copy verbose level to INTERNAL macros
	so as not to re-define user-defined parameters */
#ifdef XGIS_LOG_LEVEL_DEBUG5
#	define XGIS_LOG_LEVEL_INTERNAL_DEBUG5
#endif
#ifdef XGIS_LOG_LEVEL_DEBUG4
#	define XGIS_LOG_LEVEL_INTERNAL_DEBUG4
#endif
#ifdef XGIS_LOG_LEVEL_DEBUG3
#	define XGIS_LOG_LEVEL_INTERNAL_DEBUG3
#endif
#ifdef XGIS_LOG_LEVEL_DEBUG2
#	define XGIS_LOG_LEVEL_INTERNAL_DEBUG2
#endif
#ifdef XGIS_LOG_LEVEL_DEBUG1
#	define XGIS_LOG_LEVEL_INTERNAL_DEBUG1
#endif
#ifdef XGIS_LOG_LEVEL_INFO
#	define XGIS_LOG_LEVEL_INTERNAL_INFO
#endif
#ifdef XGIS_LOG_LEVEL_WARNING
#	define XGIS_LOG_LEVEL_INTERNAL_WARNING
#endif
#ifdef XGIS_LOG_LEVEL_ERROR
#	define XGIS_LOG_LEVEL_INTERNAL_ERROR
#endif

#ifndef XGIS_LOG_LEVEL_NO_IMPLICIT_HIGHER_LEVELS
	/* if a verbose level is enabled, all higher levels are as well 
		supported levels are:
		none
		DEBUG5 - DEBUG1
		INFO
		WARNING
		ERROR
		FATAL - this one cannot be disabled, so no need to define it */
#	ifdef XGIS_LOG_LEVEL_INTERNAL_DEBUG5
#		define XGIS_LOG_LEVEL_INTERNAL_DEBUG4
#	endif
#	ifdef XGIS_LOG_LEVEL_INTERNAL_DEBUG4
#		define XGIS_LOG_LEVEL_INTERNAL_DEBUG3
#	endif
#	ifdef XGIS_LOG_LEVEL_INTERNAL_DEBUG3
#		define XGIS_LOG_LEVEL_INTERNAL_DEBUG2
#	endif
#	ifdef XGIS_LOG_LEVEL_INTERNAL_DEBUG2
#		define XGIS_LOG_LEVEL_INTERNAL_DEBUG1
#	endif
#	ifdef XGIS_LOG_LEVEL_INTERNAL_DEBUG1
#		define XGIS_LOG_LEVEL_INTERNAL_INFO
#	endif
#	ifdef XGIS_LOG_LEVEL_INTERNAL_INFO
#		define XGIS_LOG_LEVEL_INTERNAL_WARNING
#	endif
#	ifdef XGIS_LOG_LEVEL_INTERNAL_WARNING
#		define XGIS_LOG_LEVEL_INTERNAL_ERROR
#	endif
#endif /* XGIS_LOG_LEVEL_CASCADE */

/* public macros */
#define XGIS_LOG_FATAL(...) do {\
	XGIS_LOG_INTERNAL_PREFIX(0, "FATAL");\
	fprintf(XGIS_LOG_FILE, __VA_ARGS__);\
	XGIS_LOG_INTERNAL_SUFFIX(0, "FATAL");\
	exit(-42);\
} while(0)

#ifdef XGIS_LOG_LEVEL_INTERNAL_ERROR
#	define XGIS_LOG_ERROR(...) do {\
		XGIS_LOG_INTERNAL_PREFIX(1, "ERROR");\
		fprintf(XGIS_LOG_FILE, __VA_ARGS__);\
		XGIS_LOG_INTERNAL_SUFFIX(1, "ERROR");\
	} while(0)
#else
#	define XGIS_LOG_ERROR(...) do { ; } while (0)
#endif /* XGIS_LOG_LEVEL_INTERNAL_ERROR */

#ifdef XGIS_LOG_LEVEL_INTERNAL_WARNING
#	define XGIS_LOG_WARNING(...) do {\
		XGIS_LOG_INTERNAL_PREFIX(2, "Warning");\
		fprintf(XGIS_LOG_FILE, __VA_ARGS__);\
		XGIS_LOG_INTERNAL_SUFFIX(2, "Warning");\
	} while(0)
#else
#	define XGIS_LOG_WARNING(...) do { ; } while (0)
#endif /* XGIS_LOG_LEVEL_INTERNAL_WARNING */

#ifdef XGIS_LOG_LEVEL_INTERNAL_INFO
#	define XGIS_LOG_INFO(...) do {\
		XGIS_LOG_INTERNAL_PREFIX(3, "Info");\
		fprintf(XGIS_LOG_FILE, __VA_ARGS__);\
		XGIS_LOG_INTERNAL_SUFFIX(3, "Info");\
	} while(0)
#else
#	define XGIS_LOG_INFO(...) do { ; } while (0)
#endif /* XGIS_LOG_LEVEL_INTERNAL_INFO */

/* and now, all the debug levels
	if anybody knows how to do this without
	repeating the stuff for each debug level, let me know :-) */
#ifdef XGIS_LOG_LEVEL_INTERNAL_DEBUG1
#	define XGIS_LOG_DEBUG1(...) do {\
		XGIS_LOG_INTERNAL_PREFIX(4, "Debug1");\
		fprintf(XGIS_LOG_FILE, __VA_ARGS__);\
		XGIS_LOG_INTERNAL_SUFFIX(4, "Debug1");\
	} while(0)
#else
#	define XGIS_LOG_DEBUG1(...) do { ; } while (0)
#endif /* XGIS_LOG_LEVEL_INTERNAL_DEBUG1 */

#ifdef XGIS_LOG_LEVEL_INTERNAL_DEBUG2
#	define XGIS_LOG_DEBUG2(...) do {\
		XGIS_LOG_INTERNAL_PREFIX(5, "Debug2");\
		fprintf(XGIS_LOG_FILE, __VA_ARGS__);\
		XGIS_LOG_INTERNAL_SUFFIX(5, "Debug2");\
	} while(0)
#else
#	define XGIS_LOG_DEBUG2(...) do { ; } while (0)
#endif /* XGIS_LOG_LEVEL_INTERNAL_DEBUG2 */

#ifdef XGIS_LOG_LEVEL_INTERNAL_DEBUG3
#	define XGIS_LOG_DEBUG3(...) do {\
		XGIS_LOG_INTERNAL_PREFIX(6, "Debug3");\
		fprintf(XGIS_LOG_FILE, __VA_ARGS__);\
		XGIS_LOG_INTERNAL_SUFFIX(6, "Debug3");\
	} while(0)
#else
#	define XGIS_LOG_DEBUG3(...) do { ; } while (0)
#endif /* XGIS_LOG_LEVEL_INTERNAL_DEBUG3 */

#ifdef XGIS_LOG_LEVEL_INTERNAL_DEBUG4
#	define XGIS_LOG_DEBUG4(...) do {\
		XGIS_LOG_INTERNAL_PREFIX(7, "Debug4");\
		fprintf(XGIS_LOG_FILE, __VA_ARGS__);\
		XGIS_LOG_INTERNAL_SUFFIX(7, "Debug4");\
	} while(0)
#else
#	define XGIS_LOG_DEBUG4(...) do { ; } while (0)
#endif /* XGIS_LOG_LEVEL_INTERNAL_DEBUG4 */

#ifdef XGIS_LOG_LEVEL_INTERNAL_DEBUG5
#	define XGIS_LOG_DEBUG5(...) do {\
		XGIS_LOG_INTERNAL_PREFIX(8, "Debug5");\
		fprintf(XGIS_LOG_FILE, __VA_ARGS__);\
		XGIS_LOG_INTERNAL_SUFFIX(8, "Debug5");\
	} while(0)
#else
#	define XGIS_LOG_DEBUG5(...) do { ; } while (0)
#endif /* XGIS_LOG_LEVEL_INTERNAL_DEBUG5 */



#endif /* __XGISLIB_XGISLOG_H__ */
