#include "config.h"

#include "xmlparser.h"
#include "xputils.h"

XmlParser::XmlParser(void) {
	doc = NULL;
}

XmlParser::~XmlParser(void) {
	if(NULL != doc)
		xmlFreeDoc(doc);
}

bool XmlParser::parse(std::string uri) {
	if(NULL != doc)
		xmlFreeDoc(doc);

	doc = xmlParseFile(uri.c_str());
	if(doc == NULL) {
		XGIS_LOG_ERROR("Document not parsed successfully. \n");
		return false;
	}

	return true;
}

xmlNodePtr XmlParser::getChild(std::string name) {
	if (doc != NULL)   {
		xmlNodePtr root = xmlDocGetRootElement(doc);
		return getChild(root, name);
	}
	return NULL;
}

xmlNodePtr XmlParser::getChild(xmlNodePtr parent, std::string name) {
	xmlNodePtr cur = NULL;
	xmlNodePtr root = parent;
	const xmlChar *cname = (const xmlChar *)name.c_str();
	cur = root->xmlChildrenNode;
	while (cur != NULL) {
		if ((!xmlStrcmp(cur->name, cname))) {
			return cur;
		}
		cur = cur->next;
	}

	return NULL;
}

std::string XmlParser::getNodeAttrValue(std::string nodeName,
	std::string attrName) {
	xmlChar* a;
	std::string ret = "";

	if (doc != NULL) {
		xmlNodePtr n = getChild(nodeName);

		if (n != NULL) {
			a = xmlGetProp(n, (const xmlChar *)attrName.c_str());
			ret = std::string((char*)a);
			xmlFree(a);
		}
	}
  
	return ret;
}

void XmlParser::traverseNodeVals(xmlNodePtr root, std::string nodeName,
	std::string attrName, std::vector<std::string> &list) {
	xmlNodePtr cur = root->xmlChildrenNode;

	while (cur != NULL) {
		if ((!xmlStrcmp(cur->name, (const xmlChar *)nodeName.c_str()))) {
			xmlChar *a = xmlGetProp(cur, (const xmlChar *)attrName.c_str());
			XGIS_LOG_DEBUG5("%s\n",a);
			std::string s;
			s.erase();
			s.assign((char*)a);
			list.push_back(s);
			xmlFree(a);
		}
		traverseNodeVals(cur, nodeName, attrName, list);
		cur = cur->next;
	}    
}

int XmlParser::getListNodeAttrVals(std::string nodeName, std::string attrName,
	std::vector<std::string> &list) {
	int ret = -1;
	list.clear();
	
	if (doc != NULL) {
		xmlNodePtr root = xmlDocGetRootElement(doc);
		traverseNodeVals(root, nodeName, attrName, list);
		ret = (int)list.size();
	}
	return ret;
}
