#include "config.h"

#include "xputils.h"

/* marjan: these two are *required* for linux build, don't remove them! ;) */
#include <iostream>
#include <assert.h>
#include <time.h>
#include <stdlib.h>
#include <string.h>

#ifdef WIN32
#  define WIN32_LEAN_AND_MEAN
#  include <windows.h>
#else
#  include <sys/types.h>
#  include <dirent.h>
#endif /* WIN32 */

#ifdef HAVE_ERRNO_H
#  include <errno.h>
#endif /* HAVE_ERRNO_H */

#ifdef HAVE_UNISTD_H
#  include <unistd.h>
#endif /* HAVE_UNISTD_H */

#ifdef HAVE_SYS_STAT_H
#  include <sys/stat.h>
#endif /* HAVE_SYS_STAT_H */


#ifdef WIN32 
	const char dirSlash = '\\';
#else
	const char dirSlash = '/';
#endif /* WIN32 */

#ifdef WIN32
static LPWSTR xpConvertStringToW(const std::string& dir) {
	LPWSTR wdir = (LPWSTR)new wchar_t[dir.size()+1];
	int succ = MultiByteToWideChar(CP_ACP, 0, dir.c_str(), (int)dir.size()+1, wdir, (int)dir.size()+1);
	if(!succ) {
		DWORD errNum = GetLastError();
		XGIS_LOG_ERROR("error converting to wchar buffer: ");
		if( errNum == ERROR_INSUFFICIENT_BUFFER ) {
			XGIS_LOG_ERROR("insufficient buffer\n");
		} else if( errNum == ERROR_INVALID_FLAGS ) {
			XGIS_LOG_ERROR("invalid flags\n");
		} else if( errNum == ERROR_INVALID_PARAMETER ) {
			XGIS_LOG_ERROR("invalid parameters\n");
		} else {
			XGIS_LOG_ERROR("unknown error\n");
		}
		return NULL;
	}

	return wdir;
}

#define EPOCHFILETIME (116444736000000000LL)
int gettimeofday(struct timeval *tv, struct timezone *tz) {
	/* copied from
		http://www.winehq.org/pipermail/wine-devel/2003-June/018082.html
	*/
	assert(tz == NULL);
	assert(tv != NULL);

    FILETIME        ft;
    LARGE_INTEGER   li;
    __int64         t;

    GetSystemTimeAsFileTime(&ft);
    li.LowPart  = ft.dwLowDateTime;
    li.HighPart = ft.dwHighDateTime;
    t  = li.QuadPart;       /* In 100-nanosecond intervals */
    t -= EPOCHFILETIME;     /* Offset to the Epoch time */
    t /= 10;                /* In microseconds */
    tv->tv_sec  = (long)(t / 1000000);
    tv->tv_usec = (long)(t % 1000000);
    return 0;
}

void usleep(unsigned int useconds)
{
	if (useconds > 500)
		Sleep ((useconds+500)/1000);
	else if (useconds > 0)
		Sleep (1);
	else
		Sleep (0);
}
#endif /* WIN32 */

void xpPrintTimeOfDay(FILE *f) {
	timeval tv;
	gettimeofday(&tv, NULL);
	time_t seconds = tv.tv_sec;
	struct tm *t = localtime(&seconds);
#	if 0
		//with year & date
		fprintf(f, "%04d-%02d-%02d %02d:%02d:%02d.%03d",
			t->tm_year+1900, t->tm_mon+1, t->tm_mday,
			t->tm_hour, t->tm_min, t->tm_sec, tv.tv_usec/1000);
#	elif 0
		//no year, with date
		fprintf(f, "%02d-%02d %02d:%02d:%02d.%03d",
			t->tm_mon+1, t->tm_mday,
			t->tm_hour, t->tm_min, t->tm_sec, tv.tv_usec/1000);
#	else
		//no date
		fprintf(f, "%02d:%02d:%02d.%03ld",
			t->tm_hour, t->tm_min, t->tm_sec, tv.tv_usec/1000);
#	endif /* 0/1 */
}

#ifdef WIN32
#	include <float.h>
#endif

float __xgislib_create_nan() {
	//nan defined in a smart-ass way to avoid "possible division of zero" warning
	float nan = (float)rand();
	nan -= nan;
	nan /= nan;
	assert(isnan(nan));
	return nan;
}

std::string xpGetCurrentDirectory() {
#ifdef WIN32
	const int buffSize = 1024;
	LPWSTR buffer = new WCHAR[buffSize];
	char* str = new char[buffSize];

	DWORD currDirSize = GetCurrentDirectory(buffSize, buffer);
	if( !currDirSize ) {
		//error getting tmp dir
		//buffer could be to small
		DWORD errNum = GetLastError();
		XGIS_LOG_ERROR("Could not get temp directory, error num %d\n", errNum);
		return "";
	}

	int res = WideCharToMultiByte(CP_ACP, 0, buffer, -1, str, buffSize-1, 0, 0);
	if(!res) {
		DWORD errNum = GetLastError();
		XGIS_LOG_ERROR("Error converting wchar buffer: ");
		if( errNum == ERROR_INSUFFICIENT_BUFFER ) {
			XGIS_LOG_ERROR("insufficient buffer\n");
		} else if( errNum == ERROR_INVALID_FLAGS ) {
			XGIS_LOG_ERROR("invalid flags\n");
		} else if( errNum == ERROR_INVALID_PARAMETER ) {
			XGIS_LOG_ERROR("invalid parameters\n");
		}
		str[0] = '\0';
	}
	std::string currDir(str);

	delete buffer;
	delete str;

	return currDir;
#else
	int buffSize = 1024;
	std::string rv;

	do {
		char *str = new char[buffSize + 1];
		
		if(NULL == getcwd(str, buffSize)) {
			if(ERANGE == errno)
				buffSize *= 2;
			else {
				buffSize = 0;
				rv = "";
			}
		}
		else {
			rv = str;
			buffSize = 0;
		}
		
		delete str;
	} while(buffSize > 0);
	
	return rv;
#endif /* WIN32 */
}

std::string xpMakePathAbsolute(const std::string& path) {
#ifdef WIN32
	std::string dir = "";
	if(path[0] == '\\') {
		//relative path given
		dir = xpGetCurrentDirectory();
		dir += path;
	} else if( path[1] == ':' && path[2] == '\\') {
		//abs path given
		dir = path;
	} else {
		//relative path is given with just a name
		dir = xpGetCurrentDirectory();
		dir += "\\";
		dir += path;            
	}

	return dir;
#else
	std::cerr << "xpMakeAbsolute not implemented." << std::endl;
	assert(0);
#endif /* WIN32 */
}

/*
* returns path to the temp directory without the trailing slash/backslash
*/
std::string xpGetTempDirectory(void) {
#ifdef WIN32
	const int buffSize = 1024;
	LPWSTR buffer = new WCHAR[buffSize];
	char* str = new char[buffSize];

	DWORD tmpDirSize = GetTempPath(buffSize, buffer);
	if( !tmpDirSize ) {
		//error getting tmp dir
		//buffer could be to small
		DWORD errNum = GetLastError();
		XGIS_LOG_ERROR("Could not get temp directory, error num %d\n", errNum);
		return "";
	}

	//delete trailing backslash
	if (buffer[wcslen(buffer)-1] == '\\')
		buffer[wcslen(buffer)-1] = '\0';

	int res = WideCharToMultiByte(CP_ACP, 0, buffer, -1, str, buffSize-1, 0, 0);
	if(!res) {
		DWORD errNum = GetLastError();
		XGIS_LOG_ERROR("error converting wchar buffer ");
		if( errNum == ERROR_INSUFFICIENT_BUFFER ) {
			XGIS_LOG_ERROR("insufficient buffer\n");
		} else if( errNum == ERROR_INVALID_FLAGS ) {
			XGIS_LOG_ERROR("invalid flags\n");
		} else if( errNum == ERROR_INVALID_PARAMETER ) {
			XGIS_LOG_ERROR("invalid parameters\n");
		} else {
			XGIS_LOG_ERROR("unknown error\n");
		}
		str[0] = '\0';
	}
	std::string tmpDir(str);

	delete buffer;
	delete str;

	return tmpDir;
#else
	/* linux and other unices */
	char *tmp = getenv("TEMP");
	if(NULL != tmp)
		return tmp;
	else
		return "/tmp";
#endif /* WIN32 */
}

bool xpMakeDir(const std::string &path) {
#ifdef WIN32
	std::string dir = xpMakePathAbsolute(path);

	//check if directory exists
	if( xpPathExists(dir) ) {
		XGIS_LOG_WARNING("Path exists\n");
		return true;
	}

	LPWSTR wdir = xpConvertStringToW(dir);

	int succ = CreateDirectory(wdir, NULL);

	delete wdir;

	if( !succ ) {
		DWORD err_id = GetLastError();
		if( err_id == ERROR_PATH_NOT_FOUND ) {
			XGIS_LOG_ERROR("Could not create directory: path not found\n");
		} else {
			XGIS_LOG_ERROR("Unknown error while creating directory\n");
		}
		return false;
	}       

	return true;
#else
	return (mkdir(path.c_str(), 0777) == 0);
#endif /* WIN32 */
}

bool xpIsDirectory(const std::string &path) {
#ifdef WIN32
	//convert path to absolute path if needed
	std::string dir = xpMakePathAbsolute(path);
	LPWSTR wpath = xpConvertStringToW(dir);

	DWORD res = GetFileAttributes(wpath);

	delete wpath;

	if(!res || res == INVALID_FILE_ATTRIBUTES) {
		//file or dir not exists
		return false;
	}

	return (res == FILE_ATTRIBUTE_DIRECTORY);
#else
	struct stat sbuf;
	if(0 == stat(path.c_str(), &sbuf)) {
		return (S_ISDIR(sbuf.st_mode) != 0);
	}
	return false;
#endif /* WIN32 */
}

bool xpIsFile(const std::string &path) {
#ifdef WIN32
	//convert path to absolute path if needed
	std::string dir = xpMakePathAbsolute(path);
	LPWSTR wpath = xpConvertStringToW(dir);

	DWORD res = GetFileAttributes(wpath);

	delete wpath;

	if(!res || res == INVALID_FILE_ATTRIBUTES) {
		//file or dir not exists
		return false;
	}

	return ( res != FILE_ATTRIBUTE_DIRECTORY );
#else
	struct stat sbuf;
	if(0 == stat(path.c_str(), &sbuf)) {
		return (S_ISREG(sbuf.st_mode) != 0);
	}
	return false;
#endif /* WIN32 */
}

bool xpPathExists(const std::string &path) {
#ifdef WIN32
	//convert path to absolute path if needed
	std::string dir = xpMakePathAbsolute(path);
	LPWSTR wpath = xpConvertStringToW(dir);

	DWORD res = GetFileAttributes(wpath);

	delete wpath;

	if(!res || res == INVALID_FILE_ATTRIBUTES) {
		//file or dir not exists
		return false;
	}

	if( res != FILE_ATTRIBUTE_DIRECTORY ) {
		return false;
	}

	return true;
#else
	struct stat sbuf;
	return (0 == stat(path.c_str(), &sbuf));
#endif /* WIN32 */
}

int xpGetFileSize(const std::string &path) {
#ifdef WIN32
	LPWSTR fname = xpConvertStringToW(path);

	HANDLE hfile = CreateFile(fname,    
		GENERIC_READ,          
		FILE_SHARE_READ,       
		NULL,                  
		OPEN_EXISTING,         
		FILE_ATTRIBUTE_NORMAL, 
		NULL); 

	delete fname;

	if( hfile == INVALID_HANDLE_VALUE ) {
		//the file does not exists
		CloseHandle(hfile);
		return -1;
	}

	DWORD ref;
	DWORD succ = GetFileSize(hfile, &ref);

	CloseHandle(hfile);

	return (int)succ;
#else
	struct stat sbuf;
	if(0 == stat(path.c_str(), &sbuf))
		return sbuf.st_size;
	else
		return -1;	
#endif /* WIN32 */
}

bool xpDeleteFile(const std::string &path) {
#ifdef WIN32
	LPWSTR fname = xpConvertStringToW(path);
	BOOL res = DeleteFile(fname);
	delete fname;
	if(!res) {
		XGIS_LOG_ERROR("Error deleting file\n");
		return false;
	}
	return true;
#else
	return (0 == unlink(path.c_str()));
#endif /* WIN32 */
}

int xpProcessDirectory(const char *dir, XPDirEntryProcessor proc, void *closure) {
#ifdef WIN32
	assert("xpProcessDirectory NOT IMPLEMENTED on WIN32 platform." == NULL);
	return -1;
#else
	int n = 0;
	DIR *d = opendir(dir);

	if(NULL == d) {
		return -1;
	}

	struct dirent *de;
	while(NULL != (de = readdir(d))) {
		if(0 == strcmp(".", de->d_name) || 0 == strcmp("..", de->d_name)) {
			continue;
		}
		n++;
		if(!proc(dir, de->d_name, closure)) {
			break;
		}
	}
	
	closedir(d);

	return n;
#endif /* WIN32 */
}
