#ifndef __SOCALIB_XPUTILS_H__
#define __SOCALIB_XPUTILS_H__

#include <config.h>
#include <assert.h>
#include <stdio.h>
#include <math.h>
#include <string>

// cross-platform portable functions in order to make code readable *and*
// compileable on Linux, OSX and winblows

#ifdef WIN32
#	define isnan(A) _isnan(A)
#	define snprintf _snprintf
#endif /* WIN32 */

extern const char dirSlash;

typedef bool (*XPDirEntryProcessor)(const char *dir, const char *entryPath, void *closure);

extern std::string xpGetTempDirectory(void);
extern bool xpMakeDir(const std::string &path);
extern bool xpIsDirectory(const std::string &path);
extern bool xpIsFile(const std::string &path);
extern bool xpPathExists(const std::string &path);
extern int xpGetFileSize(const std::string &path);
extern bool xpDeleteFile(const std::string &path);
extern int xpProcessDirectory(const char *dir, XPDirEntryProcessor proc, void *closure);

extern void xpPrintTimeOfDay(FILE *f);

#include "xgislog.h"

#ifdef WIN32
	/* WIN32 doesn't have gettimeofday() or usleep(), so we provide our own;
	*	first, we need the timeval struct, which is defined in winsock2.h
	*/
	#include <winsock2.h>
	
	/* the tz parameter is not supported and must be NULL! */
	extern int gettimeofday(struct timeval *tv, struct timezone *tz);
	extern void usleep(unsigned int useconds);
#else
#	include <time.h>
#	include <sys/time.h>
#	include <unistd.h>
#endif /* WIN32 */


#ifndef MAX
#  define MAX(a,b) (((a)>(b))?(a):(b))
#endif /* !MAX */

#ifndef MIN
#  define MIN(a,b) (((a)<(b))?(a):(b))
#endif /* !MAX */

#ifndef CLAMP
#  define CLAMP(a, m, M) (((a)<(m))?(m):(((a)>(M))?(M):(a)))
#endif /* !CLAMP */  

#ifndef M_PI
#  define M_PI 3.14159265358979323846
#endif /* M_PI */

extern float __xgislib_create_nan();
#ifndef NAN
#	define NAN __xgislib_create_nan()
#endif /* NAN */

// string functions
#ifndef WIN32
#	include <strings.h>
#	define stricmp(a, b) strcasecmp(a, b)
#	define strnicmp(a, b, n) strncasecmp(a, b, n)
#endif /* !WIN32 */

// networking stuff
#ifdef WIN32

#	define XPSocket SOCKET;
#	define XP_SOCKET_OK(a)                    ((a) != INVALID_SOCKET)
#	define XP_SOCKET_RV_OK(a)                 ((a) != SOCKET_ERROR)
#	define XP_SOCKET_RV_IO_OK(a)              ((a) > 0)
#	define XP_SOCKET_RV_AGAIN(n)              ((n) == SOCKET_ERROR && WSAGetLastError() == WSAEWOULDBLOCK)
#	define XP_SOCKET_RV_CLOSE(n)              ((n) == 0)
#	define xpSocketRead(s, b, l)              ::recv((s), (b), (l), 0)
#	define xpSocketWrite(s, b, l)             ::send((s), (b), (l), 0)
#	define xpSocketClose(s)                   ::closesocket((s))
#	define xpSocketSelect(s, f1, f2, f3, tv)  ::select(NULL, (f1), (f2), (f3), (tv))    

#else

#	define XPSocket int;
#	define XP_SOCKET_OK(a)                    ((a) >= 0)
#	define XP_SOCKET_RV_OK(a)                 ((a) >= 0)
#	define XP_SOCKET_RV_IO_OK(a)              ((a) > 0)
#	define XP_SOCKET_RV_AGAIN(n)              ((n) < 0 && errno == EAGAIN)
#	define XP_SOCKET_RV_CLOSE(n)              ((n) == 0)
#	define xpSocketRead(s, b, l)              ::read((s), (b), (l))
#	define xpSocketWrite(s, b, l)             ::write((s), (b), (l))
#	define xpSocketClose(s)                   ::close((s))
#	define xpSocketSelect(s, f1, f2, f3, tv)  ::select((s) + 1, (f1), (f2), (f3), (tv))    

#endif /* WIN32 */

#endif /* __SOCALIB_XPUTILS_H__ */
