#include "config.h"

#ifdef WIN32
#	define _WINSOCKAPI_
#	include <windows.h>
#endif

#if defined (GLUT_HEADERS_IN_GLUT)
#	include <GLUT/glut.h>
#elif defined (GLUT_HEADERS_IN_GL)
#	include <GL/glut.h>
#else
#	include <glut.h>
#endif

#include "glutui.h"

static GlutUI *instance = NULL;

double GlutUI::lastTime = 0;
int GlutUI::lastButton = -1;

osg::observer_ptr<osgViewer::GraphicsWindow> GlutUI::window = NULL;

// GLUT callbacks
// they just pass control over to OSG
void GlutUI::display(void) {
	
	// update and render the scene graph
	if (instance->viewer.valid()) {
		// limit FPS to max
		instance->viewer->frameLimiter();
		// draw frame
		instance->viewer->frame();
	}
	
	// Swap Buffers
	glutSwapBuffers();
	glutPostRedisplay();
}

void GlutUI::reshape(int w, int h) {
	// update the window dimensions, in case the window has been resized.
	if (window.valid()) {
		window->resized(window->getTraits()->x,	window->getTraits()->y, w, h);
		window->getEventQueue()->windowResize
			(window->getTraits()->x, window->getTraits()->y, w, h );
	}
}

void GlutUI::mousebutton(int button, int state, int x, int y) {
	if (window.valid()) {
		if (state == 0) {
			if ((button == GLUT_LEFT_BUTTON) || (button == GLUT_RIGHT_BUTTON)) {
				double now = window->getEventQueue()->getTime();
				bool doubleClick = ((lastButton == button) && (now - lastTime <= 0.5));
				if(doubleClick) {
					window->getEventQueue()->mouseDoubleButtonPress(x, y, button + 1);
				}
				else {
					window->getEventQueue()->mouseButtonPress(x, y, button + 1);
				}
				lastButton = button;
				lastTime = now;
			}
			else {
#ifdef GLUT_HAS_WHEEL_EVENTS
				osgGA::GUIEventAdapter::ScrollingMotion sm;
				if ( button == GLUT_WHEEL_UP )
					sm = osgGA::GUIEventAdapter::SCROLL_UP;
				else if ( button == GLUT_WHEEL_DOWN )
					sm = osgGA::GUIEventAdapter::SCROLL_DOWN;
				
				window->getEventQueue()->mouseScroll(sm);
#endif /* GLUT_HAS_WHEEL_EVENTS */
			}
		}
		else {
			if ((button==GLUT_LEFT_BUTTON)||(button==GLUT_RIGHT_BUTTON)) {
				window->getEventQueue()->mouseButtonRelease(x, y, button + 1);
			}
			else {
#ifdef GLUT_HAS_WHEEL_EVENTS
				osgGA::GUIEventAdapter::ScrollingMotion sm;
				if ( button == GLUT_WHEEL_UP )
					sm = osgGA::GUIEventAdapter::SCROLL_UP;
				else if ( button == GLUT_WHEEL_DOWN )
					sm = osgGA::GUIEventAdapter::SCROLL_DOWN;
				
				window->getEventQueue()->mouseScroll(sm);
#endif /* GLUT_HAS_WHEEL_EVENTS */
			}
		}
	}
}

void GlutUI::mousemove(int x, int y) {
	if (window.valid()) {
		window->getEventQueue()->mouseMotion( x, y );
	}
}

unsigned int GlutUI::getOsgModKeyMask() {
	int modes = glutGetModifiers();
	unsigned int rv = 0;
	if (modes & GLUT_ACTIVE_SHIFT) 
		rv |= osgGA::GUIEventAdapter::MODKEY_SHIFT;
	if (modes & GLUT_ACTIVE_CTRL)
		rv |= osgGA::GUIEventAdapter::MODKEY_CTRL;
	if (modes & GLUT_ACTIVE_ALT)
		rv |= osgGA::GUIEventAdapter::MODKEY_ALT;
	return rv;
}

void GlutUI::sendKey(int key) {
	osg::ref_ptr<osgGA::GUIEventAdapter> keyDown = new osgGA::GUIEventAdapter();
	keyDown->setEventType(osgGA::GUIEventAdapter::KEYDOWN);
	keyDown->setKey(key);
	keyDown->setModKeyMask(getOsgModKeyMask());

	osg::ref_ptr<osgGA::GUIEventAdapter> keyUp = new osgGA::GUIEventAdapter(*keyDown);
	keyUp->setEventType(osgGA::GUIEventAdapter::KEYUP);

	window->getEventQueue()->addEvent(keyDown.get());
	window->getEventQueue()->addEvent(keyUp.get());
}

void GlutUI::keyboard(unsigned char key, int x, int y) {
	if (window.valid())
		sendKey((osgGA::GUIEventAdapter::KeySymbol)key);
}

void GlutUI::specialKey(int key, int x, int y) {
	if (window.valid()) 	{
		osg::ref_ptr<osgGA::GUIEventAdapter> keyEvent = new osgGA::GUIEventAdapter();
		switch (key) {
		case GLUT_KEY_LEFT:
			sendKey(osgGA::GUIEventAdapter::KEY_Left);
			break;
		case GLUT_KEY_RIGHT:
			sendKey(osgGA::GUIEventAdapter::KEY_Right);
			break;
		case GLUT_KEY_UP:
			sendKey(osgGA::GUIEventAdapter::KEY_Up);
			break;
		case GLUT_KEY_DOWN:
			sendKey(osgGA::GUIEventAdapter::KEY_Down);
			break;
		case GLUT_KEY_PAGE_DOWN:
			sendKey(osgGA::GUIEventAdapter::KEY_Page_Down);
			break;
		case GLUT_KEY_PAGE_UP:
			sendKey(osgGA::GUIEventAdapter::KEY_Page_Up);
			break;
		default:
			return;
		}

		
	}
}

// void GlutUI::registerTimerCallback(TimerCallback cb, int ms, int arg) {
//	glutTimerFunc(ms, cb, arg);
//}
void GlutUI::setupViewer() {
	// link osg viewer to window
	getViewer();
	window = viewer->setUpViewerAsEmbeddedInWindow(0, 0, width, height);
	setupUI();
}

void GlutUI::run() {
	if(NULL != instance) {
		return;
	}
	instance = this;

#ifdef XGIS_ASYNC_STARTUP
	startup();
#endif /* XGIS_ASYNC_STARTUP */

	glutInit(&argc, argv);
	
	// create window
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGBA | GLUT_DEPTH | GLUT_ALPHA);
	glutInitWindowPosition(0, 0);
	glutInitWindowSize(width, height);
	glutCreateWindow(title.c_str());

	// register callbacks
	glutDisplayFunc(display);
	glutReshapeFunc(reshape);
	glutMouseFunc(mousebutton);
	glutMotionFunc(mousemove);
	glutKeyboardFunc(keyboard);
	glutSpecialFunc(specialKey);

	postInitialized();

#ifndef XGIS_ASYNC_STARTUP
	startup();
#endif /* XGIS_ASYNC_STARTUP */
	
	glutMainLoop();
}
