#ifndef __OBSCENE_HUD_H__
#define __OBSCENE_HUD_H__

#include <osgText/Text>

#include <xgislib/world.h>
#include <xgislib/resourcebag.h>

class HUD : public osg::Group {
protected:
	osg::ref_ptr<osg::Viewport> viewport;
	osg::ref_ptr<World> world;
	osg::Timer timer;

	/* map */
	osg::PositionAttitudeTransform *mapCursorTransform;
	float mapQuadSize;
	double mapDim;

	/* compass */
	osg::MatrixTransform *orientationTransform;
	
	/* info display */
	osg::Geode *textGeode;
	osgText::Text *textGaussKr, *textLatLon, *textWorld, *textSpeedAlt, *textTasks;
	
	/* position transforms for parts of HUD */
	osg::PositionAttitudeTransform *textPosition;
	osg::PositionAttitudeTransform *mapPosition;
	osg::PositionAttitudeTransform *compassPosition;
	
	/* status bar */
	osg::ref_ptr<osgText::Text> statusText;
	osg::Geode *statusGeode;
	double statusTimeout;
	bool statusVisible;
	double statusBarFontSize;

	OpenThreads::Mutex statusMutex;
	std::string newStatusMessage;
	unsigned int newStatusTimeout;	
	
	void createMap(World *world);
	void createCompass(World *world);
	void createStatusBar(World *world, ResourceBag *resourceBag);
	void createInfoDisplay(World *world, ResourceBag *resourceBag);

	int getWorldEdgeVertices(World *world, float mapQuadSize,
		osg::Vec3Array *vertices);

	void updateTextPosition();

public:
	HUD(osg::Viewport *_viewport, World *_world, ResourceBag *_resourceBag);

	void updateTasks(int pendingTasks);
	void updatePosition(double speed, const osg::Vec3 &position);
	void updateCompass(double angle);
	void updateStatus();
	
	void setStatus(const std::string &text, unsigned int timeout = 5);

	void setViewport(osg::Viewport *_viewport);
	
	Coords::GaussKr map2world(float mapX, float mapY);
};

#endif /* __OBSCENE_HUD_H__ */
