#ifndef __OBSCENE_OBSVIEWER_H__
#  define __OBSCENE_OBSVIEWER_H__ 

#include <osgViewer/Viewer>

#include "hud.h"

#define HUD_CAMERA_NAME    "hudcam"
#define ENV_CAMERA_NAME    "envcam"

class ObsViewer : public osgViewer::Viewer {
	friend class PickHandler;
	friend class ZoomHandler;
	friend class ObsceneManipulator;
	
protected:
	
	static const float maxFieldOfView, minFieldOfView;
	void zoom(float zoomFactor);
	
	// protect the constructor in order to prevent explicit instantiation
	// due to singleton pattern!
	virtual ~ObsViewer(void) {}

	// frame limiting
    osg::Timer flTimer;
    osg::Timer_t flNow;
    osg::Timer_t flLast;
	int maxFps;
	
	// custom near/far plane computation
	float farPlane;
	
	// viewport dimensions
	float width, height;
	
	// slave cameras
	osg::ref_ptr<osg::Camera> hudCam, envCam;
	osg::ref_ptr<osg::Group> env;
	osg::ref_ptr<HUD> hud;

	osg::Camera *createSlaveCamera(const char *name,
		osg::Camera::RenderOrder order, int pos, double near, double far);
	void setupSlaveCamera(osg::Camera *cam);

public:
	ObsViewer(float _farPlane);

	// adding cameras
	void addHUD(HUD *_hud) {
		hudCam->addChild(_hud);
		hud = _hud;
	}

	osg::Camera *getHUDCamera() {
		return hudCam.get();
	}

	void setEnvironment(osg::Group *_env) {
		envCam->addChild((osg::Node *)_env);
		env = _env;
	}
	
	void setupSlaveCameras();
	void syncSlaveCameras();
	
	void resizeViewer(int x, int y, int w, int h);
	
	// frame limiting
	void frameLimiter(void);
	void setMaxFPS(int _maxFps) {
		maxFps = _maxFps;
	}
	int getMaxFPS() {
		return maxFps;
	}

};

#endif /* __OBSCENE_OBSVIEWER_H__ */
