#ifndef __XGISVIEWER_XGISUI_H__
#	define __XGISVIEWER_XGISUI_H__

#include <osg/Group>

#include <OpenThreads/Thread>

#include <xgislib/semaphore.h>
#include <xgislib/resourcebag.h>
#include <xgislib/gpstrack.h>
#include <xgislib/world.h>

#include "obsviewer.h"
#include "obscenemanipulator.h"
#include "hud.h"

class XGisUI {
protected:
	osg::ref_ptr<World> world;
	osg::ref_ptr<osg::Group> root;
	osg::ref_ptr<ObsViewer> viewer;
	osg::ref_ptr<HUD> hud;
	osg::ref_ptr<ObsceneManipulator> manipulator;
	
	int argc;
	char **argv;
	int width;
	int height;
	int fps;
	int startFrame;
	std::string title;
	std::string imageDir;
	
	ObsViewer *getViewer();

	Semaphore uiInitSemaphore, startupSemaphore;
	OpenThreads::Thread *startupThread;
	
public:
	void postInitialized() { uiInitSemaphore.post(); }
	void postStartedUp() { startupSemaphore.post(); }

	virtual void setup(int _argc, char **_argv,
		int _width, int _height, int _fps, int startFrame,
		const std::string &_title, const std::string &_imageDir,
		OpenThreads::Thread *_startupThread);
	virtual void setupUI();
	virtual void startup();
	virtual void setScene(osg::Group *root, World *world, HUD *hud);

	virtual void setupViewer() = 0;
	virtual void run() = 0;
	
	World *getWorld() {
		return world.get();
	}
	
	ObsceneManipulator *getManipulator() {
		return manipulator.get();
	}
};

class XGisUIFactory {
	static XGisUI *instance;

	XGisUIFactory();
	
public:
	static XGisUI *getInstance();
};

#endif /* !__XGISUI_H__*/
